/**
 * An agent to make all the necessary calls to Google Analytics
 */
var analyticsAgent = (function() {
	/*
	 * A name list is use to replace the hashname( from pure index.html) with a customer defined name 
	 * which is use to display in Google Analytics
	 */
	var nameList = {
		"detail-page":	function(name){
			// check MyStore
			if(typeof $("#detail-address-make-default").css("background-color") != "undefined" &&
				$("#detail-address-make-default").css("background-color")	== "rgb(237, 140, 34)"){
				return "my-store-page";
			}
			return "locator-detail-page";
		},
		"clinic-detail-page": function(name){
			// check MyStore
			if(typeof $("#clinic-detail-address-make-default").css("background-color") != "undefined" &&
				$("#clinic-detail-address-make-default").css("background-color")	== "rgb(199, 34, 56)"){
				return "my-clinic-page";
			}
			return "clinic-detail-page";
		},
		"record-page":	"medication-record-page",
		"direction-page": function(name){
			console.log($("#lower-header").attr('class'));
			if($("#lower-header").hasClass("lower-header-open")){	// Locator direction page
				return "locator-direction-page";
			}else{									// Clinic direction page
				return "clinic-direction-page";
			}
		},
		"clinic-list-view": "list-view",
		"clinic-map-view": 	"map-view",
		"clinic-search-filter":	 "search-filter",
		"shopping-input-add": "add item",  
        "shopping-manage-list-showAll": "manage list[show all]",
        "shopping-manage-list-showComplete": "manage list[show complete]",
        "shopping-manage-list-showIncomplete": "manage list[show incomplete]", 
        "shopping-manage-list-deleteAll": "manage list[delete all]",
        "shopping-manage-list-deleteComplete": "manage list[delete complete]", 
        "shopping-manage-list-deleteIncomplete": "manage list[delete incomplete",
        "shopping-destroy": "delete item",
	};
	
	/* Replace the input name with the value in nameList if input name is a key in nameList,
	 * return the original name otherwise.
	 */
	var replaceName = function(name){
		if(nameList.hasOwnProperty(name)){ 
			if(typeof nameList[name] == "function"){
				return nameList[name](name);	// return function result
			}else{
				return nameList[name]; 		// Found name to replace
			}
		}else{ 
			return name; 				// no name to replace
		}
	};
	
	/*
	 * A track list containing all the events and corresponding DOM element's ids need to be track:
	 * 
	 * EXAMPLE: locatorTrackList["page"] = ["map-view", "list-view", "show-direction", "search-filter"];
	 * - locatorTrackList: name of the page tracking list, used to insert into global trackList.
	 * - "page": event category
	 * - ["map-view", "list-view", "show-direction", "search-filter"]: list of element's id that will trigger the tracking call
	 * 
	 * To track more events, add more events/elements' id to existing tracking list or create a new page tracking list and add events to it 
	 */
	var trackList = {};
	
	/* locator page's tracking list */
	var locatorTrackList = {};
	locatorTrackList["page"] = ["map-view", "list-view", "show-direction", "search-filter"];
	locatorTrackList["phone"] = [];
	trackList["#locator-page"] = locatorTrackList;	// Add self to global tracking list
	
	/* clinic page's tracking list */
	var clinicTrackList = {};
	clinicTrackList["page"] = ["clinic-map-view", "clinic-list-view", "clinic-show-direction", "clinic-search-filter"];
	clinicTrackList["phone"] = [];
	trackList["#clinic-page"] = clinicTrackList;	// Add self to global tracking list
	
	/* detail page's tracking list */
	var detailTrackList = {};
	detailTrackList["phone"] = [];
	trackList["#detail-page"] = detailTrackList;
	
	/* clinic detail page's tracking list */
	var clinicDetailTrackList = {};
	clinicDetailTrackList["phone"] = [];
	trackList["#clinic-detail-page"] = clinicDetailTrackList;
	
	/* refill page's tracking list */
	var refillTrackList = {};
	refillTrackList["page"] = ["refill-submit",];
	trackList["#refill-page"] = refillTrackList;
	
	/* transfer page's tracking list */
	var transferTrackList = {};
	transferTrackList["page"] = ["transfer-showTransferFrom", 
	                             "transfer-backPatientInfo", "transfer-showTransferTo", 
	                             "transfer-backTransferFrom", "transfer-showRequest", 
	                             "transfer-backTransferTo", "transfer-form-submit"];
	trackList["#transfer-page"] = transferTrackList;
	
	/* Shopping page's tracking list */
	var shoppingTrackList = {};
	shoppingTrackList["page"] = ["shopping-input-add", 
	                             "shopping-manage-list-showAll", "shopping-manage-list-showComplete",
	                             "shopping-manage-list-showIncomplete", "shopping-manage-list-deleteAll",
	                             "shopping-manage-list-deleteComplete", "shopping-manage-list-deleteIncomplete",];
	trackList["#shopping-page"] = shoppingTrackList;
	
	/* Public functions could be invoke by outside program */
	return {
		
		/*
		 * Create a new GA tracker for this device 
		 * NOTE: this function should only be called within main.js->onDeviceReady()
		 */
		gaCreateTracker: function(){
			ga('create', getGATrackingId(), {	// getGATrackingId() returns the tracking ID of Google Analytics and is defined in Keys.js
			  	'storage': 'none',
			  	'clientId':device.uuid
			});	
		},
		
		/* 
		 * Send active pageview to Google Analytics
		 * NOTE: this function should only be called within AppView.js->initialize()
		 * 
		 * Input:
		 * 		hashTag: 	current page's hash tag
		 */
		gaSendPage: function(hashTag){
			ga('send', 'pageview', {'page': hashTag.length==0 ? 'homePage' : replaceName(hashTag.replace('#', ''))});	// Send hash tag without '#' prefix
		},
		
		/* 
		 * Send tap event on doms included within trackList
		 * NOTE: this function should only be called within main.js->$(document).on("tap") listener
		 * 
		 * Input:
		 * 		event: 		Jquery event object
		 * 		hashTag: 	current page's hash tag
		 */
		gaSendEvent: function(event, hashTag){
			if(hashTag in trackList){
				var target = $(event.target);	// Retrieve event target
				
				/* Phone Call event, should check this event FIRST, since it only need to check the target element */
				if("phone" in trackList[hashTag]){
					if(target.is("a") && target.attr("href").indexOf("tel:") >= 0){
						ga('send', 'event', 'Phone Call', 'tap', "Call from ("+replaceName(hashTag.replace("#",""))+")");
						return;	
					}
				}
				/* Subpage event */
				if("page" in trackList[hashTag]){
					target.parents().andSelf().map(function() {	//?? maybe parentsUtil() would be a better choice here??, also consider to use while loop for early stop
						/* Check id name */
						id = $(this).attr("id");		// Get the parent's id
						if(trackList[hashTag]["page"].indexOf(id) >= 0){	// Search through tracking list
							// Send the matched element id
							ga('send', 'event', 'Sub-Page', 'tap', replaceName(id)+"("+hashTag.replace("#","")+")");
							return;
						}
					});
				}

				/* Shopping list event */
//				if("shop" in trackList[hashTag]){
//					if(trackList[hashTag]["shop"].indexOf(target.attr("id"))){
//						ga('send', 'event', 'Shopping', 'tap', "Add a item to shopping list");
//						return;	
//					}
//				}
			}
		}
		
	}// End return 


})();