/**
* Handlerbar helper registration
* 
* $$ Consider to put this part in a seperated place $$
*/
Handlebars.registerHelper('list', function(items, options) {
    var out = "<ul>";

    for (var i = 0, l = items.length; i < l; i++) {
        out = out + "<li>" + items[i] + "</li>";
    }

    return out + "</ul>";
});

Handlebars.registerHelper('PharmacistTitleSub', function(item, options) {
    if (item.indexOf("Pharmacy Manager") != -1 && item.indexOf("Lic") != -1) {
        var ind = item.indexOf('#');
        return item.substring(ind + 1);
    }
    else {
        return "";
    }
});

/* Helper to display store hours if exist or CLOSED otherwise */
Handlebars.registerHelper('storeHours', function(open, close) {
    if (open && close) {
        return open + " - " + close;
    }
    else {
        return "CLOSED";
    }
});

/* Helper to parse out the extra attributes after "|" in StoreServices */
Handlebars.registerHelper('each', function(item, options) {
//	 console.log("Handlebars helper {{each}}: item is :" + item);
	 var ret = "";
	 for(var i=0, j=this.length; i<j; i++) {
		  if(this[i].indexOf("|")!= -1){
			  ret = ret + options.fn(this[i].substring(0,this[i].indexOf("|"))); 
		  }else{
			  ret = ret + options.fn(this[i]);
		  }
	  }
	  return ret;
});

/* Prefix number with zeros */
Handlebars.registerHelper('prefixZeros', function(num, len) {
    num = num + "";
//    console.log("Handlebars helper {{prefixZeros}}: num:" + num + "(length:"+num.length+") len:"+len);
    if(num.length < len){
        var pre = new Array(len - num.length + 1).join('0');
        return pre + num;
    }else{
        return num;
    }
});

/**
* Initailizing Model's and View 
*/

var deviceOSVersion;

/* Current Store */
var currentStoreView = new CurrentStoreView();
/* Current Clinic */
var currentClinicView = new CurrentClinicView();

/* Login */
var janrainLogin = new JanRainLogin();	/* Default is not login */
var loginView = new LoginView({model: janrainLogin});

/* Set up timer for Activity Timeout */
function ActivityTimeout () {
      var lastActivity = Date.now();
      var thisActivity = Date.now();
      var timeOutMinutes = 15; // in minutes
      var timeOutValue = timeOutMinutes * 60 * 1000;
    return {
        getLastActivity: function() {
     		alert(lastActivity);
      },
        setLastActivity: function() {
     		thisActivity = Date.now();
     		if (window.localStorage.getItem("JanRainLogin"))
     		{
     			if ((thisActivity - lastActivity) > timeOutValue)
     			{
     				loginView.model.logout();
					var id ="";
					$.mobile.changePage(id, {transition: 'slide'});
			   		setTimeout(function() {
          			// just in case we are being called from resume, jAlert is wrapped in a setTimeout
					jAlert("Your session has timed out.","Please Sign in again");
        			}, 0);
     			}
     	 }
     	lastActivity = thisActivity; 
      }
    }
}
var activityTimeout = ActivityTimeout();

/* check if we need to clear email and password */
$(document).on("pagebeforeshow", "#login-page", function(event) {
    if ($("#onoff").val() == 0) {
        $("#login-email").val("");
        $("#login-password").val("");
    }
});
$(document).on("pageshow", "#login-page", function(event) {
	var str = navigator.userAgent;
	if (str.indexOf("Chrome") != -1)
		{
			$(".rexall-button-xlarge").removeClass('rexall-kitkat-grey');
		}
});

/* reset password */
var resetPassword = new ResetPassword();
var resetPasswordView = new ResetPasswordView({ model: resetPassword });

/* Setting */
var communicationView = new CommunicationView({model: janrainLogin});
var securityView = new SecurityView({model: janrainLogin});

/* Profile and Medication Records */
var profile = new Profile();
var profileView = new ProfileView({model: profile});

/* Enrolment */
var enrolment = new Enrolment();

profileView.loginModel = janrainLogin;
/* Set login model for janrain update */
var recordListView = new RecordsListView({model: profile});

/* Make sure medication record page is scrollable */ 
$(document).on("pageshow", "#record-page", function(event) {
	var str = navigator.userAgent;
	if (str.indexOf("Chrome") != -1)
		{
			$(".rexall-button-xlarge").removeClass('rexall-kitkat-grey');
		}
    recordListView.scroller.refresh();
});

/* Prescription Transfer */
var transferView = new TransferView({model: profile});

/* Shopping */
var shoppingList = new ShoppingList();
var shoppingListView = new ShoppingListView({collection: shoppingList});

/* Refill Page */
var refillView = new RefillView();

/* Make sure refill page is scrollable */ 
$(document).on("pageshow", "#refill-page", function(event) {
	var str = navigator.userAgent;
	if (str.indexOf("Chrome") != -1)
		{
			$(".rexall-button-xlarge").removeClass('rexall-kitkat-grey');
		}
    refillView.refillScroll.refresh();
});

/* Contact us Page */
var contactUsView = new ContactUsView();

/* Store Locator */
var storeFilterView = new StoreFilterView();/* Store Search Service */
var storeDetailPage = new StoreDetailView();
var storeDetailList = new StoreDetailList();	/* Fetch store data */

storeFilterView.setModel(storeDetailList);

var storeListMapView = new StoreListMapView({"collection" : storeDetailList}); /* has to create before appView.... need to work on this one later.... maybe should create every subview before appView*/
var storeListView = new StoreListView({"collection" : storeDetailList});	
var storeLocatorView = new StoreLocatorView({"collection" : storeDetailList});

/* Work around for issue 1002 - 1004 */
var directionPage = new StoreDirectionView();

$(document).on("pageshow", "#direction-page", function(event) {
    if (appView && appView.showClinicDirection) {
        /* Show clinic direction */
        $("#clinic-show-direction").removeClass("locator-menu-hide");
        $("#my-clinic").addClass("locator-menu-hide");
		
        directionPage.model = clinicStoreDetailPage.model;
        directionPage.render();
        /* Search by default, from search location */
        var lt = clinicDetailList.lat;
        var lg = clinicDetailList.lng;
        directionPage.displayDirection(lt, lg);
    }
    else {
        /* Show menu bar, under the situation of back from refill page */
        $("#lower-header").removeClass("lower-header-close").addClass("lower-header-open");
	
        /* Show direction */
        /* reset direction page */
        directionPage.model = storeDetailPage.model;
        directionPage.render();
        /* Search by default, from search location */
        var lt = storeDetailList.lat;
        var lg = storeDetailList.lng;
        directionPage.displayDirection(lt, lg);
    }
});

var refillStoreListView = new RefillStoreListView({"collection" : storeDetailList});

/* Clear filters before loading the Store Locator page */
//$(document).on("pagebeforeshow", "#locator-page", function (event) {
//    $("[name='filterCheckbox']").attr("checked", false);
//    storeFilterView.serviceList = [];
//    console.log("clear filter");
//});

/* Clinic Locator */
var clinicFilterView = new ClinicFilterView();
var clinicStoreDetailPage = new ClinicStoreDetailView();
var clinicDetailList = new ClinicDetailList();
/* Fetch store data */
clinicFilterView.setModel(clinicDetailList);

var clinicListView = new ClinicListView({"collection" : clinicDetailList});
var clinicListMapView = new ClinicListMapView({"collection" : clinicDetailList});
var clinicLocatorView = new ClinicLocatorView({"collection" : clinicDetailList});

/* Pharmacy Service , DEPRECATED, NEED TO REMOVE LATER
var serviceView = new PharmacyServiceView();
var bloodServiceView = new ServiceView({"serviceView":serviceView, "el": $("#service-blood"), "filter" : "Blood Pressure Screening"});
var fluShotServiceView = new ServiceView({"serviceView":serviceView, "el": $("#service-flu-shot"), "filter" : "Flu Shot"});
var clinicServiceView = new ServiceView({"serviceView":serviceView, "el": $("#service-clinic"), "filter" : "Medical Clinic"});
var quitSmokeServiceView = new ServiceView({"serviceView":serviceView, "el": $("#service-quit-smoke"), "filter" : "Ready-2-Quit Smoking Program"});
*/

/* Settings page */
var settingView = new SettingsView();

/* Pharmacy page */
var pharmacyView = new PharmacyView();
$(document).on("pageshow", "#pharmacy-page", function(event) {
	var str = navigator.userAgent;
	if (str.indexOf("Chrome") != -1)
		{
			$(".rexall-button-xlarge").removeClass('rexall-kitkat-grey');
		}
    pharmacyView.refresh();
});

/* Term of use page */
var termOfUseView = new TermOfUseView();
/* Privacy Policy page */
var privacyPolicyView = new PrivacyPolicyView();
var enrolmentView = new EnrolmentView();
var enrolmentFormView = new EnrolmentFormView();
var enrolmentTermView = new EnrolmentTermView();
var enrolmentCompleteView = new EnrolmentCompleteView();
var enrolmentRequestView = new EnrolmentRequestView();
var enrolmentWaitView = new EnrolmentWaitView();
var enrolmentConfirmView = new EnrolmentConfirmView();
var enrolmentVerifyView = new EnrolmentVerifyView();
var enrolmentMedicationInfoView = new EnrolmentMedicationInfoView();

$(document).on("pageshow", "#enrolment-page", function(event) {
    enrolmentView.scroll.refresh();
});
$(document).on("pageshow", "#enrolment-form-page", function(event) {
    enrolmentFormView.scroll.refresh();
});
$(document).on("pageshow", "#enrolment-term-page", function(event) {
    enrolmentTermView.scroll.refresh();
});

/* APP */
var appView = new AppView();

/* Some view need to call AppView's function */
refillView.setApp(appView);
loginView.setApp(appView);
storeLocatorView.setApp(appView);
clinicLocatorView.setApp(appView);
resetPasswordView.setApp(appView);
//storeListItemView.setApp(appView);

/**
*  Router 
*/

var router = new MainRouter();

Backbone.history.start();

/**
* Phone gap deviceready setups
*/

function onBackKeyDown() {
    appView.goBack();
};
function onResume() {
	activityTimeout.setLastActivity();
};
 
/* PhoneGap is loaded and it is now safe to make calls PhoneGap methods */
function onDeviceReady() {
    /* Register the backbutton event listener */
    document.addEventListener("backbutton", onBackKeyDown, false);
    document.addEventListener("menubutton", onBackKeyDown, false);
    document.addEventListener("resume", onResume, false);
    document.addEventListener("active", onResume, false);
    /* Show splash screen */
    navigator.splashscreen.hide();
    /* Check network connection */
    appView.checkConnection();
    
	var str = navigator.userAgent;
	if (str.indexOf("Chrome") == -1)
		{
			$(".kitkat-fix").addClass('rexall-not-kitkat');
			$(".rexall-button-xlarge").addClass('rexall-not-kitkat-grey');
		}
		else
		{
			$(".kitkat-fix").addClass('rexall-kitkat');
			$(".rexall-button-xlarge").blur(function() {
				$(".rexall-button-xlarge").removeClass('rexall-kitkat-grey');
				});
			$(".rexall-button-xlarge").click(function() {
				$(".rexall-button-xlarge").addClass('rexall-kitkat-grey');
				});
		}
	//var deviceOS  = device.platform;  //fetch the device operating system
    deviceOSVersion = device.version;  //fetch the device OS version

    // Input field mask
    setHTML5DateFormat("transfer-birth-date");
    setHTML5DateFormat("refill-birth-date");
    setHTML5DateFormat("enrolment-request-birthdate");
    setHTML5DateFormat("enrolment-confirm-birthdate");
    setInputmaskPostalCode("contact-us-zip");
    setInputmaskPostalCode("transfer-patient-postal");

    // Google Analytics setup connection
    if(typeof analyticsAgent != 'undefined') {
        analyticsAgent.gaCreateTracker();	// Create a new tracker
    }

      /*alert("Device OS: " + deviceOS); 
         alert("Device OS Version: " + deviceOSVersion); 
         
     if(parseFloat(deviceOSVersion).toFixed(1)>=4.3)
     {
    	 alert("Device OS Version is >=4.3 ");    	 
     }else{
    	 alert("Device OS Version  is < 4.3" );
     }*/
	
};

/* Phone Gap Setup */
document.addEventListener("deviceready", onDeviceReady, false);

function storeSubstring(store_str,len){
	if(len&&store_str&&store_str.length>len){
		store_str = store_str.substring(0,len)+'...';
	}
	return store_str
};

/**
 * Jquery events delegation
 */
$(document).on("tap",function(event) {
	 activityTimeout.setLastActivity();
	 /* Google Analytics Track specific DOM element being tapped */
	 if(typeof analyticsAgent != 'undefined'){
		 analyticsAgent.gaSendEvent(event, location.hash);
	 }
});


/**
 * Hover events handling for mobile device
 */
$(document).on("vmousedown", function(event){
	$(event.target).closest(".menu-option").addClass("highlight");	// Home page icon and Settings page sub menu
	$(event.target).closest(".rexall-kitkat").addClass("highlight");// Pharmacy and Settings page menu option
});
$(document).on("vmouseup", function(event){
	$(event.target).closest(".menu-option").removeClass("highlight");
	$(event.target).closest(".rexall-kitkat").removeClass("highlight");
});

/**
 * Utility functions for format checking of input fields
 */

/* Place holder work around */
//$("input[type='number']").each(function(i, el) {
//    el.type = "text";
//    el.onfocus = function(){
//        this.type="number";
//    };
//    el.onblur = function(){this.type="text";};
//});

/* Date of birth format checking for version >= 4.3 */
function setHTML5DateFormat43(s){
	
	document.getElementById(s).setAttribute("type","tel");	// Set input type
	
	/*
	 * A handler need to bind to the date field after the user blur.
	 */
	var blurHandler = function(e){	// Final check for the value of dob
		var birthDate = getHTML5Date(s);
		if($(e.target).val().length<10){		// Completion check
			jAlert("Please enter valid date of birth", "Please Try Again");
		}
//		}else if(!appView.checkDateOfBirthAfter18(birthDate)){	// 18 year old check
//			jAlert("You must be at least 18 years of age.", "Please Try Again");
//			$(e.target).val("");
//		}
		return false;
	}
	
	/*
	 * Show error message of the date during real time input
	 */
	var showDateAlert = function(target, msg) {
		var cb = function(rs) {		// A callback function bind to the "close" button of the alert box
			target.focus();			// Focus to the date field again after closing alert box
			//window.KeyBoard.showKeyBoard();	// Show soft keyboard
			//target.on("blur", blurHandler);	// Bind blur handler back to the field
		};
		target.off("blur");			// Unbind blur handler(**need to unbind all blur event in 4.4, it somehow duplicate the handler if you bind multiple times) before showing alert box since the "close" button will trigger handler functions
		jAlert(msg, "Please Try Again", cb);// Show alert box
	}
	
	/* Setup key up events */
	$("#"+s).off("keyup").keyup(function (e) {		// Set keyup listener
			var key = e.charCode || e.keyCode || 0;	// Detect backspace
			var target = $(e.target);		// Get event target
			/** IMPORTANT: Once the correct behavior has been defined, this line should be refined */
			//target.on("blur", blurHandler);	// Bind blur event handler to this field once the user pressed a key
			
			var value = target.val();
			if(value.length > 10){		// Set max length
				target.val(value.substring(0,10));
				return false;
			}
			if(key == 8 && (value.length == 2 || value.length == 5)){	// eg. 12/12/ backspace-> 12/1
				value = value.substring(0, value.length-1);
			}
			
			value = value.replace(/\D/g, "");	// Filter out non-digital charactors
			len = value.length;					// Length of digital chars
			date = "";		// Date(dd) string
			dateVal = 0;	// Date value
			month = "";		// Month(mm) string
			monthVal = 0;	// Month value
			year = "";		// Year(yyyy) string
			yearVal = 0;	// Year value
			
			if(len >= 0){						// Check date
				date = value.substring(0,(len>1?2:len));
				dateVal = parseInt(date,10);	// Need to add 10 here, otherwise 08 and 09 will parse to 0
				if(len == 1 && dateVal > 3){
					date = "0"+date;
				}else if(dateVal > 31){			// Date should be less than 32 for each month
					//showDateAlert(target, "Please enter valid date.");
					date = date.substring(0,1);				
				}else if(dateVal == 0){
					if(key==8){					// If backspace hited, zero value should be cleaned since some of the zeros is added by code as a prefix, therefore android won't remeber it, and cause a backspace not working problem
						date = "";
					}else if(len==2){			// 00 -> 0
						//showDateAlert(target, "Please enter valid date.");
						date = "0";
					}
				}
			}
			
			if(len >= 3){						// Check month
				month = value.substring(2, (len>4?4:len));
				monthVal = parseInt(month,10);
				
				if((monthVal == 2 && dateVal > 29) ||
				((monthVal == 4 || monthVal == 6 || monthVal == 9 || monthVal == 11) && dateVal == 31) ||
				monthVal > 12){
					if(len == 3){				// 31/2 -> 31
						//showDateAlert(target, "Please enter valid month.");
						month = "";
					}else{						// 31/11 -> 31/1
						//showDateAlert(target, "Please enter valid month.");
						month = month.substring(0,1);
					}
				}else if(len == 3 && monthVal > 1){
					month = "0" + monthVal;
				}else if(monthVal == 0){
					if(key == 8){
						month = "";
					}else if(len == 4){			// 12/00 -> 12/0
						//showDateAlert(target, "Please enter valid month.");
						month = "0";
					}
					
				}
			}
			
			if(len >= 5){						// Check year
				year = value.substring(4, len);
				yearVal = parseInt(year);
				if(len==5 && (yearVal>2 || yearVal==0)){		// 31/11/3 -> 31/11 
					//showDateAlert(target, "Please enter valid year.");
					year = "";
				}else if(len==6 && (yearVal<19 || yearVal>20)){	// Range check should between 19xx-20xx
					//showDateAlert(target, "Please enter valid year.");
					year = year.substring(0, 1);
				}else if(len == 8){				// Check the whole year
					if(dateVal==29 && monthVal==2 && (yearVal%4 != 0)){	// Check leaf year
						//showDateAlert(target, "Please enter valid leaf year.");
						year = year.substring(0, 3);
					}
					
					var wholeDate = getHTML5Date(s);
					var tarID = target.attr("id");	// Target id use to filter the 18 year old validation
					if(!appView.checkDateOfBirthAfter18(wholeDate) &&	// 18 year old check
						tarID != "refill-birth-date" &&		// Removed validation on manual refill page
						tarID != "transfer-birth-date"){	// Removed validation on transfer page	
						
						showDateAlert(target, "You must be at least 18 years of age.");
						year = "";
					}
				}else if(yearVal == 0 && key == 8){
					//showDateAlert(target, "Please enter valid year.");
					year = "";
				}
			}
			// Assemble the formatted value
			result = "";	
			if(date.length > 0){
				result += date;
				if(date.length == 2){
					result += "/";
				}
			}
			if(month.length > 0){
				result += month;
				if(month.length == 2){
					result += "/";
				}
			}
			if(year.length > 0){
				result += year;
			}
			// Write back to input box
			target.val(result);
			
	});
}

function setHTML5DateFormat(s){

	var str = navigator.userAgent;
	//if (str.indexOf("Chrome") == -1)
//	if(!deviceOSVersion){
//		 deviceOSVersion = device.version;
//		 if(!deviceOSVersion){
//			 return;
//		 }
//	}
	setHTML5DateFormat43(s);
//	if(parseFloat(deviceOSVersion).toFixed(1)>=4.3)
//	{
//		//document.getElementById(s).setAttribute("type","date");
//		setHTML5DateFormat43(s);	// Set formating procedures for the field		
//	}else{
//		document.getElementById(s).setAttribute("type","tel");
//		document.getElementById(s).setAttribute("placeholder","dd/mm/yyyy");
//	    $("#"+s).inputmask("dd/mm/yyyy");
//	}
};

function getHTML5Date(s) {
	var str = navigator.userAgent;
	var result='';
	//if (str.indexOf("Chrome") == -1)
//	if(parseFloat(deviceOSVersion).toFixed(1)<4.3)	
//	{	
		result = $("#"+s).val();
//	}else
//	{
//		var val = $("#"+s).val().replace(/\//g,"-");
//		var regExpression = /(\d+)-(\d+)-(\d+)/g;
//		
//		var data = regExpression.exec(val);
//		if(!data||data.length<3){
//			return val;
//		}
//		result = data[3] + "/" + data[2] + "/" + data[1];
//	}
	return result;
};

function getHTML5DateLength(s) {
	var str = navigator.userAgent;
	var result='';
	//if (str.indexOf("Chrome") == -1)
//	if(parseFloat(deviceOSVersion).toFixed(1)>=4.3)
//	{
//		result = ($("#"+s).val()) ? 8:0 ; // HTML5 date field is always correct, length wise
//	}else
//	{
//		result = $("#"+s).inputmask('unmaskedvalue').length;
//	}
	result = $("#"+s).val().replace(/\D/g, "").length;
	return result;
}; 

/* Phone number format check */
$("input[for='phone-html5']").keyup(function (e) {
	var key = e.charCode || e.keyCode || 0;	// Detect backspace
	var target = $(e.target);				// Get event target
		
	var value = target.val();
	if(value.length > 14){					// Set max length
		target.val(value.substring(0,14));
		return false;
	}
	
	if(key == 8){
		// eg. (555)_ backspace-> (55
		if(value.length == 5){	value = value.substring(0, value.length-2);	}
		// eg. (555)_555- backspace -> (555)_55
		if(value.length == 9){	value = value.substring(0, value.length-1);	}
	}
	
	value = value.replace(/\D/g, "");	// Filter out non-digital charactors
	len = value.length;					// Length of digital chars
	
	var result = "";	// Initialize formatting result
	
	var a = "";		// (xxx)
	var b = "";		// _xxx
	var c = "";		// -xxxx
	
	// get first part (xxx)
	if(len >= 1){	a = value.substring(0,(len>3?3:len));	}
	// get second part _xxx
	if(len >= 4){	b = value.substring(3,(len>6?6:len));	}
	// get thrid part -xxxx
	if(len >= 7){	c = value.substring(6,len);	}
	
	// assemble parts together
	if(a.length > 0){	
		result += "(" + a;	
		if(a.length == 3){
			result += ") ";
		}
	}
	if(b.length > 0){	
		result += b;	
		if(b.length == 3){
			result += "-";
		}
	}
	if(c.length > 0){	result += c;	}
	
	// Write back to input box
	target.val(result);
});/*.blur(function(e){	// Blur check for completion
	
	var value = $(e.target).val();	// Get input value
	if(value.length != 14){			// Check length
		jAlert("Please enter valid phone number.", "Please Try Again");
		return false;
	}
});*/

/* Postal format checking for version >= 4.3 */
function setHTML5PostalCodeFormat43(s){
	
	/* Setup key up events */
	$("#"+s).off("keyup").keyup(function (e) {	// Set keyup listener
		
		var key = e.charCode || e.keyCode || 0;	// Detect backspace
		var target = $(e.target);				// Get event target
			
		var value = target.val();
		
		if(value.length > 7){					// Set max length
			target.val(value.substring(0,7));
			//return false;	// Can't return here since postal code need char AND num check which will bypass this check point, length check is done in the final assembling part.
		}
		
		if(key == 8 && value.length == 3){	// eg. a1a_ backspace-> a1
			value = value.substring(0, value.length-1);
		}
		
		value = value.replace(/ /g, "");	// Filter out all space
		len = value.length;					// Length of digital chars

		var result = "";	// Initialize formatting result
		var i;	// loop counter
		for(i=0; i<len; i++){
			
			if(i%2==0){	// even position, charactor check
				if(checkLetter(value.charAt(i))){
					result += value.charAt(i);
				}else{	break;	}
			}else{		// odd position, number check
				if(checkNumber(value.charAt(i))){
					result += value.charAt(i);
				}else{	break;	}
			}
			
		}

		len = result.length;
		if(len>=3){
			result = value.substring(0,3) + " " + value.substring(3, (len>6?6:len));	// Max length check set to be here
		}else{
			result = value.substring(0,len);
		}
		// Write back to input box
		target.val(result);
		/* Work around for tablet remembering all the invalid inputs */
		//target.blur();
		//target.focus();			
		//window.KeyBoard.showKeyBoard();
	});/*.blur(function(e){	// Blur check for completion
		
		var value = $(e.target).val();	// Get input value
		if(value.length != 7){			// Check length
			jAlert("Please enter valid postal code.", "Please Try Again");
			return false;
		}
	});*/
	
}

function checkNumber(data){
	var reg = /^[0-9]*$/;
	 return reg.test(data);
};

function checkLetter(data){
	var reg = /^[A-Za-z]+$/;
	return reg.test(data);
};
function setInputmaskPostalCode(id){
	var str = navigator.userAgent;
	//if(str.indexOf("Chrome") == -1){
//	if(parseFloat(deviceOSVersion).toFixed(1)<4.3){
//		$("#"+id).inputmask("a9a 9a9");
//	}else{
		setHTML5PostalCodeFormat43(id);
//	}
};


//a1a 1a1
/*
$("input[for='postal-jquery']").keyup(function(e){
		var str = navigator.userAgent;		
		$postal = $(this);
		//var postalVal = $postal.val();
		//$phone.val(null).val(postalVal);
		var len = $postal.val().length;
		var data = $postal.val().substr(len-1,1);
		var oldData =  $postal.val().substr(0,len-1);
		if(len > 7){	// Set max length
			$postal.val(oldData);
		}
		//if(str.indexOf("Chrome") != -1){		
		if(parseFloat(deviceOSVersion).toFixed(1)>4.3){			
			
			if(len==1||len==6){
				if(!checkLetter(data)){
					$postal.val(oldData);
					return false;
				};
			}else if(len==2||len==5||len==7){
				if(!checkNumber(data)){
					$postal.val(oldData);
					return false;
				};
			}else if(len==3){
				if(!checkLetter(data)){
					$postal.val(oldData);
					return false;
				}else{
					$postal.val($postal.val()+' ');
				};
			}
		}else if(parseFloat(deviceOSVersion).toFixed(1)==4.3){
			
			 var key =  e.charCode || e.keyCode || 0;
			 //alert(key);
			 if(key !== 8 && key !== 9){
				if(len==1||len==6){
					if(!checkLetter(data)){
							$postal.val(oldData);
							return false;
					};
				}else if(len==2||len==5||len==7){
					if(!checkNumber(data)){
						$postal.val(oldData);
						return false;
				      };
				}else if(len==3){
					if(!checkLetter(data)){
						$postal.val(oldData);
						return false;
					}else{
						$postal.val($postal.val()+' ');
					};
				}
				}
			
		}else{
			
//			var key =  e.charCode || e.keyCode || 0;
//			if(key !== 8 && key !== 9){
//				if(len==1||len==6){
//					if(!checkLetter(data)){
//						$postal.val(oldData);
//						return false;
//					};
//				}else if(len==2||len==5||len==7){
//					if(!checkNumber(data)){
//						$postal.val(oldData);
//						return false;
//					};
//				}else if(len==3){
//					if(!checkLetter(data)){
//						$postal.val(oldData);
//						return false;
//					}else{
//						$postal.val($postal.val()+' ');
//					};
//				}
//			}
		}
	}).blur(function () {
		var str = navigator.userAgent;
		//if(str.indexOf("Chrome") != -1){
		if(parseFloat(deviceOSVersion).toFixed(1)>=4.3){
			$postal = $(this);
			var len = $postal.val().length;
			var flag = true;
			if (len!=7) {
				flag&=false;
			}else{
				var value = $postal.val();
				if(!checkLetter(value.substr(0,1))||!checkLetter(value.substr(2,1))||
						!checkLetter(value.substr(5,1))){
					flag&=false;
				}
				if(!checkNumber(value.substr(1,1))||!checkNumber(value.substr(4,1))||
						!checkNumber(value.substr(6,1))){
					flag&=false;
				}
				if(value.substr(3,1)!=' '){
					flag&=false;
				}
			}
			if(!flag){
				jAlert("Postal code is illegal!",'Please Try Again');
				$postal.val('');
			}
		}
	});
*/







