var JanRainLogin = Backbone.Model.extend({
	
	/* Default setting */
	defaults: function () {
		return { hasLogin: false,
				 hasHomeStore: false
				 };
	},
	
	initialize : function () {
		/* Set the status of login */
		if(localStorage.getItem("JanRainLogin")){
			this.set({hasLogin: true});
		}else{
			this.set({hasLogin: false});
		}
		
		/* Set the status of home store */
		if(localStorage.getItem("RexallProfileHomeStore")){
			this.set({hasHomeStore: true});
		}else{
			this.set({hasHomeStore: false});
		}
		
		/* Listen to login events to update home store address */
		this.listenTo(this, "change:hasLogin", this.setHomeStoreAddress);	/* !!!!!!!!!!should be a better way to do this */
		
	},
	
	/* Validate user's input for login */
	validateInput : function (loginInput) {
		/* validate email and password */
		var email = loginInput.email;
		var password = loginInput.password;
		if(!email || !password){
			jAlert("Invalid email or password.", "Please Try Again");
			return false;
		}
		/* Set valid check for login function */
		return true;
	},
	
	/* Logout the user */
	logout: function() {
		// LocalStorage logout, this need to be remove first as loginView render() depends on ls
		localStorage.removeItem("JanRainLogin");            // remove Janrain account profile
		localStorage.removeItem("RexallProfileHomeStore");  // remove Rexall patient profile store
        localStorage.removeItem("JanRainAccessToken");      // remove Janrain access token
        localStorage.removeItem("JanRainRefreshToken");     // remove refresh token
        localStorage.removeItem("JanRainAccessTokenExpire");// remove token expire
        localStorage.removeItem("JanRainAccessTokenLastUpdate");// remove last update
        localStorage.removeItem("LoginEmail");                  // remove login email
        localStorage.removeItem("LoginPassword");               // remove password
		// Model logout
		janrainLogin.set({hasLogin: false});
		janrainLogin.set({hasHomeStore: false});
	},


    /*
        Save access token from Janrain to local storage

        Input:
        res - response returned by ajax call
        loginInput - the credential use to successfully login
     */
    saveAccessToken: function(res, loginInput){
        localStorage.setItem("JanRainAccessToken", JSON.parse(res).access_token);
        localStorage.setItem("JanRainRefreshToken", JSON.parse(res).refresh_token);
        localStorage.setItem("JanRainAccessTokenExpire", JSON.parse(res).expires_in);
        localStorage.setItem("JanRainAccessTokenLastUpdate", new Date().getTime());

        localStorage.setItem("LoginEmail", loginInput.email);
        localStorage.setItem("LoginPassword", loginInput.password);
    },

    /*
        1st step of New login method uses the only the email and password to get the access token

        Input:
        loginInput - {'email': xxxxx, 'password': xxxxx}
        loginCallBack - (optional) if provide, call this function after success
     */
    login: function (loginInput, loginCallBack) {


        /** Debug : force localstorage to remember a profile json data */
//		var data = {"result":{"security_question":"question_1","uac":null,"birthday":null,"LoyaltyNumber":"","familyName":null,"homestorenumber":"9981","profiles":[],"PVprofileID":"23068675875559956","security_answer":"snickers","id":5121,"PromotionConsent":false,"HomeProvince":"ON","avatar_url":"female_1.png","birthyear":null,"middleName":null,"changed_password":null,"emailVerified":null,"primaryAddress":{"company":null,"address2":null,"stateAbbreviation":null,"zipPlus4":null,"city":null,"address1":null,"phone":null,"zip":null,"mobile":null,"country":null},"gender":null,"lastUpdated":"2013-09-30 14:17:47.203625 +0000","PVRegistration":true,"password":{"value":"$2a$04$jptfU0.hnj58WpzAHbYyOexHv/9JEF8/16lvwIXwftYAtwm3kxZ2K","type":"password-bcrypt"},"registrationIncomplete":false,"photos":[],"email":"maguiar@propharm.com","givenName":null,"currentLocation":null,"deactivateAccount":null,"pvRegistrationIncomplete":true,"lastLogin":"2013-09-30 14:15:00 +0000","created":"2013-09-17 16:11:18.372387 +0000","healthinformation":false,"birthmonth":null,"PVRegistrationValidated":true,"displayName":"Marisa","uuid":"417e3646-df9d-4da3-9d3b-063edf2cf4b1","aboutMe":null,"display":null,"statuses":[]},"WARNING":"HTTP GET is deprecated","stat":"ok"};
//		localStorage.setItem("JanRainLogin", JSON.stringify(data.result));
//		janrainLogin.set({hasLogin: true});
//        janrainLogin.refreshToken(); // get access token
//		return;
//		/* Empty record test */
//		var data = {"result":{"security_question":"question_1","uac":null,"birthday":null,"LoyaltyNumber":"","familyName":null,"homestorenumber":"9981","profiles":[],"PVprofileID":"23068675875559956","security_answer":"snickers","id":5121,"PromotionConsent":false,"HomeProvince":"ON","avatar_url":"female_1.png","birthyear":null,"middleName":null,"changed_password":null,"emailVerified":null,"primaryAddress":{"company":null,"address2":null,"stateAbbreviation":null,"zipPlus4":null,"city":null,"address1":null,"phone":null,"zip":null,"mobile":null,"country":null},"gender":null,"lastUpdated":"2013-09-30 14:17:47.203625 +0000","PVRegistration":true,"password":{"value":"$2a$04$jptfU0.hnj58WpzAHbYyOexHv/9JEF8/16lvwIXwftYAtwm3kxZ2K","type":"password-bcrypt"},"registrationIncomplete":false,"photos":[],"email":"maguiar@propharm.com","givenName":null,"currentLocation":null,"deactivateAccount":null,"pvRegistrationIncomplete":true,"lastLogin":"2013-09-30 14:15:00 +0000","created":"2013-09-17 16:11:18.372387 +0000","healthinformation":false,"birthmonth":null,"PVRegistrationValidated":true,"displayName":"Marisa","uuid":"429f762f-c782-4eb5-9633-4ad57349e95d","aboutMe":null,"display":null,"statuses":[]},"WARNING":"HTTP GET is deprecated","stat":"ok"};
//		localStorage.setItem("JanRainLogin", JSON.stringify(data.result));
//		janrainLogin.set({hasLogin: true});
//		return;
        /** */

        // Show spinner
        $("#global-spinner").css("visibility", "visible");

        // Validate the login input
        if(!this.validateInput(loginInput))
            return false;

        // Self reference
        var self = this;

        // Login parameters
        var data = {
            "APIKey"    : getProxyAPIKey(),
            "UserName"  : loginInput.email,
            "Password"  : loginInput.password
        };

        // get Authorization code from JanRain
        $.ajax({
            url: getRexallWebserviceRoot() + "v1/AccountManagement/Entity.svc/Login/",
            type: 'GET',
            data: data,
            crossDomain: true,
            dataType: 'json',
            success: function(res) {
                if(res.Success === 1) {
                    // save access token response and login credential
                    self.saveAccessToken(res.JainRainRecord, loginInput);
                    // get user's login profile from janrain
                    if(loginCallBack){  // call back
                        loginCallBack();
                    }
                }else {     // record not found
                    // hide spinner
                    $("#global-spinner").css("visibility", "hidden");
                    $("label[for=login-email]").addClass("rexall-warning-label");
                    $("label[for=login-password]").addClass("rexall-warning-label");
                    // show error message
                    jAlert("You entered an invalid username or password.", "Please Try Again");
                }
            },
            error: function(XHR, textStatus, error) {
                // hide spinner
                $("#global-spinner").css("visibility", "hidden");
                // show error message
                jAlert("Unable to login. Please check your network connection and try again.", "Something Went Wrong");
                console.log('Error on requesting authorization code : ' + textStatus);
            }
        });
    },

    /*
        2nd step of JanRain login: get Janrain profile

        Input:
        callBack - (optional) if provide, call this function after success
     */
    getLoginProfile: function(callBack){
        // Self reference
        var self = this;

        // Check if refresh token is required
        if(this.refreshToken(function(){
            self.getLoginProfile(); // Must wrap this recursive call in a function, otherwise the value of self will change
        })){
            return true;    // return if token is refreshed and callback with this function
        }

        // Prepare find parameters
        var data = {
            "APIKey" 	    : getProxyAPIKey(),
            "AccessToken"   : localStorage.getItem("JanRainAccessToken")
        };

        $.ajax({
            url: getRexallWebserviceRoot() + "v1/AccountManagement/Entity.svc/Find/",
            type: 'GET',
            data: data,
            crossDomain: true,
            dataType: 'json',
            success: function(res) {
                if(res.Success === 1) {
                    // save login profile to local storage
                    var record = res.JainRainRecord;
                    localStorage.setItem("JanRainLogin", JSON.stringify(JSON.parse(record).result));

                    // clear out warnings
                    $("label[for=login-email]").removeClass("rexall-warning-label");
                    $("label[for=login-password]").removeClass("rexall-warning-label");

                    // call back function
                    if(callBack){
                        callBack();
                    }
                    self.set({hasLogin: true});
                }else {
                    console.log("Login failed in step 2: " + JSON.stringify(res));
                    jAlert(res.Message, "Something Went Wrong");
                }

            },
            error: function(XHR, textStatus, error) {
                // Show error message
                jAlert("Unable to login. Please check your network connection and try again.", "Something Went Wrong");
            },
            complete: function(){
                // hide spinner
                $("#global-spinner").css("visibility", "hidden");
            }
        });
    },

	/* Return the address of home store of the user */
	setHomeStoreAddress: function() {
        if (localStorage.getItem("JanRainLogin")) {
            /* Get patient's home store number for Janrain's response */
            var storeNum = JSON.parse(localStorage.getItem("JanRainLogin")).homestorenumber;

//            if (localStorage.getItem("RexallProfileHomeStore")
//                && storeNum == JSON.parse(localStorage.getItem("RexallProfileHomeStoreAddress")).StoreNumber)
//                return;			/* This home store address has already been stored in local storage */
		        
            var self = this;
            var data = {
                "APIKey" 	    : getWebserviceAPIKey(),
                "StoreNumber"   : storeNum
            };

            $.ajax({
                url: getRexallWebserviceRoot() + "v3/standard/Rexall.svc/StoreGet/",
                type: 'GET',
                data: data,
                crossDomain: true,
                dataType: 'json',
                success: function(res) {	
                    console.log("setHomeStoreAddress Callback success." + res.stat);
                    self.getHomeStoreAddressCallBack(res, self);
                },
                error: function(XHR, textStatus, error) {
                    //alert(JSON.stringify(XHR) + "\n" +textStatus +"\n" + error + "\n");
                    jAlert("An error occurs when trying to retrieve your Home Store data. Please try again.", "Something Went Wrong");
                },
                complete:function(XHR, textStatus) {
                    // self.afterFetch();
                }
            });
			
		}else{		/* Logout or not login */
			localStorage.removeItem("RexallProfileHomeStore");	/* remove ls */
		}
	},
	
	/* Callback function for .setHomeStoreAddress() webservice call */
	getHomeStoreAddressCallBack: function(data, self) {
		/* Store result to local storage */
		if(data.Store.length > 0){
			localStorage.setItem("RexallProfileHomeStore", JSON.stringify(data.Store[0]));
			self.set({"hasHomeStore": true});
		}
		
		/* Fixed j-44 (not sure if we need the below logic in other places) 
		else{
			self.set({"hasHomeStore": false});
			jAlert("Cannot find your home store in profile.", "Something Went Wrong");
		}
		*/
	},
	
	/* Update janrain profile */
	update: function(values) {

        // self reference
        var self = this;

        // Check if refresh token is required
        if(this.refreshToken(function(){
            self.update();  // Must wrap this recursive call in a function, otherwise the value of self will change
        })){
            return true;    // return if token is refreshed and callback with this function
        }

		// Prepare update parameters
		var data = {
                "APIKey" 	    : getProxyAPIKey(),
                "AccessToken"   : localStorage.getItem("JanRainAccessToken"),
                "attributes" 	: JSON.stringify(values)
        };
		
		// Update account information
		$.ajax({
	    	 url: getRexallWebserviceRoot() + "v1/AccountManagement/Entity.svc/Update/",
	    	 type: 'GET',
	    	 data: data,
	    	 crossDomain: true,
	    	 dataType: 'json',
	    	 success: function(res) {
	      	  	if(res.Success === 1) {
	      	  		/* Update local storage */
	      	  		/* Display the correct message */
					var input = JSON.stringify(values); 
	      	  		if (input.indexOf ("displayName") > -1) { 
	      	  			jAlert("Your Display Name has been updated.", "Success");
	      	  		} else if (input.indexOf ("LoyaltyNumber") > -1) { 
	      	  			jAlert("Your communication preferences have been updated.", "Success");
	      	  		} else { 
	      	  			jAlert("Your security preferences have been updated.", "Success");
	      	  		}
					
	      	  		/* refresh local profile data */
	      	  		self.refresh(values);
	            }else {
	                jAlert(res.Message, "Please try again");
	            }
	    	 },
	    	 error: function(XHR, textStatus, error) {
	    		 jAlert("Unable to update, please check your network connection and try again.", "Something Went Wrong");
	    	 }
	     });
	},
	
	/* Refresh local profile data */
	/**
	 * WIP: need to use a sync lock for sync localStorage and server data
	 * ** Lock in update(), unlock after refresh() to handle the case that disconnect during refreshing
	 */
	refresh: function(values) {

		// Self reference
        var self = this;

        // Check if refresh token is required
        if(this.refreshToken(function(){
            self.refresh(); // Must wrap this recursive call in a function, otherwise the value of self will change
        })){
            return true;    // return if token is refreshed and callback with this function
        }

		// Prepare find parameters
		var data = {
            "APIKey" 	    : getProxyAPIKey(),
            "AccessToken"   : localStorage.getItem("JanRainAccessToken"),
            "attributes"    : JSON.stringify(values)
	    };
		
		// find account information
		$.ajax({
	    	 url: getRexallWebserviceRoot() + "v1/AccountManagement/Entity.svc/Find/",
	    	 type: 'GET',
	    	 data: data,
	    	 crossDomain: true,
	    	 dataType: 'json',
	    	 success: function(res) {
                if(res.Success === 1) {
                    // Update local storage
                    var record = res.JainRainRecord;
                    localStorage.setItem("JanRainLogin", JSON.stringify(JSON.parse(record).result));
                }else{
                    jAlert(res.Message, "Please Try Again");
                }
	    	 },
	    	 error: function(XHR, textStatus, error) {
	    		 jAlert("Unable to update your local account information.", "Please Try Again");
	    	 }
	     });
		
		/* Update app views */
			/* Display name, should use better way to do this, use listener */
		if(values.displayName){
			$("#welcome").html("Hi, " + values.displayName);
		}
	}, 
	
	/* Update janrain password, need to validate current password */
	updatePassword: function(old_password, new_password) {

        // Show spinner
        $("#global-spinner").css("visibility", "visible");

        // Self reference
        var self = this;

        // Login parameters
        var data = {
            "APIKey"    : getProxyAPIKey(),
            "UserName"  : JSON.parse(localStorage.getItem("JanRainLogin")).email,
            "Password"  : old_password
        };

        // get Authorization code from JanRain
        $.ajax({
            url: getRexallWebserviceRoot() + "v1/AccountManagement/Entity.svc/Login/",
            type: 'GET',
            data: data,
            crossDomain: true,
            dataType: 'json',
            success: function(res) {
                if(res.Success === 1) {
                    // save access token, refresh token and expire threshold
                    self.saveAccessToken(res.JainRainRecord, {"email":data.UserName, "password":data.Password});
                    // send update password request
                    self.update({"password" : new_password});
                }else {     // record not found
                    // show error message
                    jAlert("The current password you entered is incorrect.", "Please Try Again");
                    $("label[for='current-password']").addClass("rexall-warning-label");
                }
            },
            error: function(XHR, textStatus, error) {
                jAlert("Unable to update your password, please check your network connection.", "Please Try Again");
            },
            complete: function(){
                // hide spinner
                $("#global-spinner").css("visibility", "hidden");
            }
        });
	},

    /* Refresh, or get the access token from JanRain server
    *
    * Input:
    * callback - (optional) should be a function, which will be called after successfully refresh token
    *
    * Output:
    * return false if no refreshToken is required
    * return true otherwise
    * */
    refreshToken: function(callback){

        // self reference
        var self = this;

        // check if access token need to be refreshed
        var expires_in = localStorage.getItem("JanRainAccessTokenExpire");
        var last_update = localStorage.getItem("JanRainAccessTokenLastUpdate")
        if((new Date().getTime() - last_update)/1000 < expires_in - 900){   // allow 15 mins buffer
            return false;   // no refresh is required for the access token
        }

        // prepare data for requesting token
        var loginInput = {
            'email': localStorage.getItem("LoginEmail"),
            'password': localStorage.getItem("LoginPassword")
        };

        self.login(loginInput, function(){
            if(callback){
                callback();
            }
        });

        return true;
    }

});


/*
 Login the user

 1st step of Login JanRain: get authorization code using user's email and password
 */
//getAuthoCode: function (loginInput) {
//
//    // Show spinner
//    $("#global-spinner").css("visibility", "visible");
//
//    // Validate the login input
//    if(!this.validateInput(loginInput))
//        return false;
//
//    // Self reference
//    var self = this;
//
//    // Login parameters
//    var data = {
//        "client_id" 			: getJanrainClientID(),
//        "client_secret"			: getJanrainSecret(),
//        "type_name"             : "user",
//        "key_attribute"         : "email",
//        "key_value"             : '"' + loginInput.email + '"',
//        "password_attribute"    : "password",
//        "password_value"        : loginInput.password,
//        "redirect_uri"          : ""    // not applicable for mobile app, leave as empty
//    };
//
//    // get Authorization code from JanRain
//    $.ajax({
//        url: getJanrainServerURL() + '/access/getAuthorizationCode',
//        type: 'POST',
//        data: data,
//        crossDomain: true,
//        dataType: 'json',
//        success: function(res) {
//            if(res.stat === "ok") {
//                var authCode = res.authorizationCode;           // retrieve authorization code from response
//                self.exchangeAuthorizationCode(authCode);   // exchange authCode for access code
//            }else {     // record not found
//                // hide spinner
//                $("#global-spinner").css("visibility", "hidden");
//                $("label[for=login-email]").addClass("rexall-warning-label");
//                $("label[for=login-password]").addClass("rexall-warning-label");
//                // show error message
//                jAlert("You entered an invalid username or password.", "Please Try Again");
//            }
//        },
//        error: function(XHR, textStatus, error) {
//            // hide spinner
//            $("#global-spinner").css("visibility", "hidden");
//            // show error message
//            jAlert("Unable to login. Please check your network connection and try again.", "Something Went Wrong");
//            console.log('Error on requesting authorization code : ' + textStatus);
//        }
//    });
//},
//
///*
// 2nd step of JanRain login: exchange authorization code for an access token
// */
//exchangeAuthorizationCode: function(authCode){
//
//    // Self reference
//    var self = this;
//
//    // request parameters
//    var data = {
//        "client_id" 			: getJanrainClientID(),
//        "client_secret"			: getJanrainSecret(),
//        "grant_type"            : "authorization_code",
//        "code"                  : authCode,
//        "redirect_uri"          : ""    // not applicable for mobile app, leave as empty
//    };
//
//    $.ajax({
//        url: getJanrainServerURL() + '/oauth/token',
//        type: 'POST',
//        data: data,
//        crossDomain: true,
//        dataType: 'json',
//        success: function(res) {
//            if(res.stat === "ok") {
//                // save access token, refresh token and expire threshold
//                self.saveAccessToken(res);
//                // get user's login profile from janrain
//                self.getLoginProfile();
//            }else {
//                jAlert("Unable to login. Please check your network connection and try again.", "Something Went Wrong");
//                console.log("Login failed in step 2: " + JSON.stringify(res));
//            }
//        },
//        error: function(XHR, textStatus, error) {
//            // hide spinner
//            $("#global-spinner").css("visibility", "hidden");
//            // show error message
//            jAlert("Unable to login. Please check your network connection and try again.", "Something Went Wrong");
//            console.log('Error on requesting access token : ' + textStatus);
//        }
//    });
//},