var Profile = Backbone.Model.extend({
		
	initialize: function () {
		/* Listen to login model */
		this.listenTo(janrainLogin, "change:hasLogin", this.handleLogin);
		
		/* Prefetch data if the user has login */
		if(janrainLogin.get("hasLogin"))
			this.preFetchData();
	},
	
	/* Fetch and remove profile data according to login and logout events, respectively */
	handleLogin: function() {
		
		if(janrainLogin.get("hasLogin") && localStorage.getItem("JanRainLogin")){
            // $("#global-spinner").css("visibility", "visible");
             this.fetchData();	/* Login, fetch data */
		}else{
			this.cleanData();	/* Logout, remove data */
		}
	},
	
	/* Actions right before pulling data */
	beforeFetch: function() {
        console.log("profile beforeFetch");
		/* Change transfer page header */
		$("#transfer-page").find(".rexall-page-header").html("Loading your profile &nbsp;&nbsp  <div id='circleG'><div id='circleG_1' class='circleG'></div><div id='circleG_2' class='circleG'></div><div id='circleG_3' class='circleG'></div></div>");
		/* Change record page header */
		$("#record-page").find(".rexall-page-header").html("Loading your profile &nbsp;&nbsp  <div id='circleG'><div id='circleG_1' class='circleG'></div><div id='circleG_2' class='circleG'></div><div id='circleG_3' class='circleG'></div></div>");
        $("#refill-to-pharmacy-address-default").val("Loading your home store, wait...");
	},
	
	/* Actions right after pulling data */
	afterFetch: function() {
        
        $("#global-spinner").css("visibility", "hidden");

		/* Change transfer page header */
		$("#transfer-page").find(".rexall-page-header").html("<p>TRANSFER PRESCRIPTIONS</p><div class='rexall-page-header-info'></div>");
		/* Change record page header */
        var record_header = $("#record-page").find(".rexall-page-header").html();

        console.log("rexall-page-header:" + record_header);

        if (loginView.fromRecord) {
          //   console.log("Inside , rexall-page-header:" + $("#record-page").find(".rexall-page-header").html());
          $("#record-page").find(".rexall-page-header").html("<p>MEDICATION RECORD</p><div class='rexall-page-header-info'></div>")
          
        }
        else
        {
             $("#record-page").find(".rexall-page-header").html("<p>REFILL USING MyRexall ACCOUNT</p><div class='rexall-page-header-info'></div>");
        }

        //udpate medication record page title store address
        router.refreshRecordPage();

        //udpate refill by prescription number refill location address
        var store = JSON.parse(localStorage.getItem("RexallProfileHomeStore"));
        if (!store)
        {
            store = localStorage.getItem("CurrentStore");
        }
        if (store)
        {
             $("#refill-to-pharmacy-address-default").val(storeSubstring(store.StoreName + ", "+store.Address + ", " + store.City+", " +store.Province,45));
          	$("#refill-form-to-default").show();
        	$("#refill-form-change-location").hide();
        }
        else
        {
            $("#refill-to-pharmacy-address-default").val("");
            $("#refill-form-to-default").hide();
        	$("#refill-form-change-location").show();
        }

        refillView.preFillBirthday();

        loginView.state = false;
        loginView.fromRefill = false;
        loginView.fromTransfer = false;
        loginView.fromRecord = false;
        loginView.fromRefillbyNumber = false;
        this.fromRefillbyNumberFromStoreLocator = false;
        
	},
	
	/* Webservice call to retrieve data from rexall API */
	fetchData: function() {

        // self reference
        var self = this;

        // Check if refresh token is required
        if(janrainLogin.refreshToken(function(){
            self.fetchData();   // Must wrap this recursive call in a function, otherwise the value of self will change
        })){
            return true;        // return if token is refreshed and callback with this function
        }

        // show spinner for record page
        if (location.hash == "record-page"){
            $("#global-spinner").css("visibility", "visible");
        }

        // prepare data
        var data = {
            "APIKey"        : getSecureWebserviceAPIKey(),
            "AccessToken"	: localStorage.getItem("JanRainAccessToken")
        };

        // prepare view change
        self.beforeFetch();

        $.ajax({
            url: getMyRexallWebserviceRoot() + "v3/secure/SecureRexall.svc/RxProfileGet/",
            type: 'GET',
            data: data,
            crossDomain: true,
            dataType: 'json',
            success: function(res) {

                console.log("Profile fetchData Callback success." + JSON.stringify(res));
                self.fetchDataCallBack(res, self);
                self.afterFetch();
            },
            error: function(XHR, textStatus, error) {
                jAlert("An error occurs when trying to retrieve your profile data, please try again.", "Something Went Wrong");
                self.afterFetch();
            },
            complete:function(XHR, textStatus) {
                
            }
        }); 
		/* Create new WebService instance */
		//var webService = new WebService(webServiceOption);
		//webService.callWebService(this.beforeFetch, this.afterFetch);
	},
	
	/* Call Back function for .fetchData() webservice call */
	fetchDataCallBack: function(data, self) {
		/* Handle service down exception */
         //console.log(JSON.stringify(data.results.result));
       // $("#global-spinner").css("visibility", "hidden");
        console.log("End profile fetchData");
		if(data.success.success == 0){
			jAlert("We cannot retrieve your profile at this time. Please try again later.", "Something Went Wrong");
			return;
		}
		// Store result to local storage
        var results = data.results;
		localStorage.setItem("RexallProfile", JSON.stringify(results.result));
		// Update self model
		self.set(results.result);
		


		/* goBack to previous page */
        setTimeout(function() {
        	if(!loginView.goToEnrolmentPageStatus){
	            if (location.hash == "#login-page" && loginView.state && self.hasRecord()) {
	                console.log("level 1 if #login-page");
	                if (loginView.fromTransfer) {
	                    var id = "#transfer-page";
	                    $.mobile.changePage(id, {transition: 'slide', changeHash: false});
	                }
	                else if (loginView.fromRecord){
	                   // var id = "#record-page";
	                   // $.mobile.changePage(id, {transition: 'slide'});
	                    console.log("Profile set medication record");
	                     $("#record-page-header").html("<p>MEDICATION RECORD</p><div class='rexall-page-header-info'></div>");
	                }
	                else{
	                   // var id = "#record-page";
	                   // $.mobile.changePage(id, {transition: 'slide'});
	                   console.log("Profile set REFILL USING MyRexall");
	                    $("#record-page-header").html("<p>REFILL USING MyRexall ACCOUNT</p><div class='rexall-page-header-info'></div>");
	                }
	            }
	            else if (location.hash === "#login-page" && loginView.state && !self.hasRecord()) {
	                appView.goBack();
	            }
        	}
            loginView.state = false;
            loginView.fromRefill = false;
            loginView.fromTransfer = false;
            loginView.fromRecord = false;
            this.fromRefillbyNumberFromStoreLocator = false;
        }, 0);
	},
	
	/* Clean up after logout */
	cleanData: function() {
		/* Remove local storage */
		localStorage.removeItem("RexallProfile");
		/* Clear out all the attributes */
		this.clear(); 
	},
	
	/* Try fetch data from localStorage */
	preFetchData: function() {
		/* Fetch from localStorage */
		var ls = localStorage.getItem("RexallProfile");
		if(ls){		/* Has localStorage */
			this.set(JSON.parse(ls));
		}else{		/* No localStorage but logged in, make a webservice call. Although this scenario should never happen */
			this.fetchData();
		}
	},
	
	/* Return boolean to indicate whether this user has medication record or not */
	hasRecord: function() {
		return this.get("prescriptions") ? true : false;
	},
	
	/* Return boolean to indicate whether this user has profile or not */
	hasProfile: function() {
		return this.get("profile") ? true : false;
	},

});