/*
 * StoreDetail model stores the detail information of an
 * individual store
 */
var StoreDetail = Backbone.Model.extend({
	
	/* Default setting of whether the store is user's preferred store,
	 * this field is determined by the value of JanRainLogin.homestorenumber and
	 * CurrentStore in Local Storage. In addition, JanRainLogin.homestorenumber has
	 * HIGHER priority then CurrentStore.
	 */
	defaults: {
		isPreferred: false,
		isPreferredClinic: false,
	},
	
	initialize: function(options) {
		
		/* Listen to self's isPreferred value change and
		 * notify parent collection of the change
		 */
		this.on("change:isPreferred", this.onPreferredChange);
		this.on("change:isPreferredClinic", this.onPreferredClinicChange);
		/* Get store number */
		//var storeNum = options.StoreNumber;
		var storeNum ;
		if(location.hash == "#clinic-page"){
			storeNum = options.ClinicNumber;
		}else{
			storeNum = options.StoreNumber;
		}

		if(!storeNum) {
			// somethinf went wrong here
			return;
		}
		
		/* Check whether is preferred with priority:
		 * JanRainLogin > CurrentStore > null
		 */
		var current;
		/*if(localStorage.getItem("JanRainLogin")){
			current = JSON.parse(localStorage.getItem("JanRainLogin")).homestorenumber;
		}else if(localStorage.getItem("CurrentStore")){
			current = JSON.parse(localStorage.getItem("CurrentStore")).StoreNumber;
		}else{
			current = null;
		}*/

		var store = JSON.parse(localStorage.getItem("RexallProfileHomeStore"));
		if(!store){
			store = JSON.parse(localStorage.getItem("CurrentStore"));
		}
		if(store){
			current = store.StoreNumber;
		}else{
			current = null;
		}

		
		if(current && storeNum == current){
			this.setPreferred();
		}else{
			this.resetPreferred();
		}

        var CurrentClinicNum = null;
        if (localStorage.getItem("CurrentClinic")) {		/* Has current store */
           CurrentClinicNum = JSON.parse(localStorage.getItem("CurrentClinic")).ClinicNumber;
        }

        console.log("CurrentClinicNum:" + CurrentClinicNum + " Store Number:" + storeNum);
        if(CurrentClinicNum && storeNum == CurrentClinicNum){
			this.setPreferredClinic();
             console.log("set setPreferredClinic " + storeNum);
		}else{
			this.resetPreferredClinic();
		}
		
       // console.log(JSON.stringify(options));
	},
	
	/* Check if this store has online refill service */
	hasRefill: function() {
		return this.get("StoreServices").indexOf("Online Refill") >= 0;
	},
	
	/* Set the isPreferred field */
	setPreferred: function() {
		this.set({'isPreferred': true});
	},
	
	/* Set the isPreferredClinic field */
	setPreferredClinic: function() {
		this.set({'isPreferredClinic': true});
	},
	
	/* Unset the isPreferred field */
	resetPreferred: function() {
		this.set({'isPreferred': false});
	},

    /* Unset the isPreferred field */
	resetPreferredClinic: function() {
		this.set({'isPreferredClinic': false});
	},
	
	/* Check and refresh the isPreferred attibute of this model 
	 * input store is the store number need to be compare with
	 */
	checkPreferred: function(store) {
		
		if(!store) return;
		
		if(this.StoreNumber == store)
			this.setPreferred();
		else
			this.resetPreferred();
	},
	
	/* Notify parent collection the change of isPreferred attribute.
	 * This will also work with the situation that multiple preferred store is allowed
	 */
	onPreferredChange: function() {
		if(!this.collection || this.get("isPreferred") === false){
			return;
		}
		
		/* reset current preferredStore */
		if(this.collection.preferredStore){
			this.collection.preferredStore.resetPreferred();
		}
		
		/* Change this.collection.preferredStore to array if multiple store is allowed */
		this.collection.preferredStore = this;		
	},
	
    onPreferredClinicChange: function() {
		if(!this.collection || this.get("isPreferredClinic") === false){
			return;
		}
		
        console.log(this.collection.preferredClinic);
		/* reset current preferredStore */
		if(this.collection.preferredClinic){
			this.collection.preferredClinic.resetPreferredClinic();
            
		}
		
		/* Change this.collection.preferredStore to array if multiple store is allowed */
		this.collection.preferredClinic = this;		

        console.log(this.collection.preferredClinic);
	},

	/* Set reference to parent collection */
	setCollection: function(c){
		this.collection = c;
	}
});














