/*
 * Collection of StoreDetail model
 */
var StoreDetailList = Backbone.Collection.extend({
	
	initialize: function(options) {
		this.self = this;
		
		/* Listen to login events to update current store */
		this.listenTo(janrainLogin, "change:hasHomeStore", this.setPreferredStore);	
		this.storeStartIndex = 0;
	},
	
	model: StoreDetail,
	
	/* Set preferred store with local storage */
	setPreferredStore: function() {
        
		/* Unset current preferred store */
		if(this.preferredStore)
        {
			this.preferredStore.resetPreferred();
            
        }
        if (storeDetailPage.model)
        {
            storeDetailPage.model.resetPreferred();
        }
        

		/* Get correct store number */
		var storeNum = null;
		if(localStorage.getItem("RexallProfileHomeStore")){	/* Has medication store */
			storeNum = JSON.parse(localStorage.getItem("RexallProfileHomeStore")).StoreNumber;
		}else if(localStorage.getItem("CurrentStore")){		/* Has current store */
			storeNum = JSON.parse(localStorage.getItem("CurrentStore")).StoreNumber;
		}
		
		/* Exception check */
		if(storeNum)
        {
		    /* Update */
		    var newStore = this.findWhere({StoreNumber: storeNum});
		
		    /* set model's isPreferred attribute */
		    if(newStore){
			    /* This change will notify collection to change preferredStore */
			    newStore.setPreferred();	
		    }else{
			    this.preferredStore = null;
		    }
        }
			
		storeDetailPage.render();		
	},
	fetchMoreCallBack: function(data, self) {
        /* Hide spinner */
		$("#global-spinner").css("visibility", "hidden");
		$("#locator-search-input").attr("readOnly",false);
		/* Set start point */
		//var newStartIndex = self.length;	/* self.length - 1 + 1 */
       var newStartIndex = this.storeStartIndex;
		var l = data.Store.length-1;
		var updated = false;
        var storeCount = 0;
        var current = this;

        console.log("fetchMoreCallBack store count:" + data.Store.length);
		/* populate data */
       if(data.Success == 1) {	/* Success on StoreList webservice call */ 
			for ( var i = newStartIndex; i <= l; i++ ) {

               data.Store[i].StoreServices=data.Store[i].StoreServices.sort();
				/* Filter the store and pharmacy hours */
				if(storeFilterView.serviceList.indexOf("Stores Currently OPEN") >= 0
				&& storeFilterView.serviceList.indexOf("Pharmacies Currently OPEN") >= 0){
					/* Search for Both open */
					if(self.storeIsOpen(data.Store[i]) && self.pharmacyIsOpen(data.Store[i])  && self.IsStoreHasFilterServices(data.Store[i].StoreServices)){
						/* Create model and set it's collection to self */
						self.create(data.Store[i]).setCollection(self);
                       updated = true;
                       storeCount++;
                     
					}
				}else if(storeFilterView.serviceList.indexOf("Stores Currently OPEN") >= 0){
					/* Search for store open */
					if(self.storeIsOpen(data.Store[i]) && self.IsStoreHasFilterServices(data.Store[i].StoreServices)){
						/* Create model and set it's collection to self */
                       self.create(data.Store[i]).setCollection(self);
                       updated = true;
                       storeCount++;
					}
				}else if(storeFilterView.serviceList.indexOf("Pharmacies Currently OPEN") >= 0){
					/* Search for store open */
					if(self.pharmacyIsOpen(data.Store[i]) && self.IsStoreHasFilterServices(data.Store[i].StoreServices)){
						/* Create model and set it's collection to self */
						self.create(data.Store[i]).setCollection(self);
                       updated = true;
                      storeCount++;
					}
				}else{
					/* Create model and set it's collection to self */
                   if (self.IsStoreHasFilterServices(data.Store[i].StoreServices))
                   {
					    self.create(data.Store[i]).setCollection(self);
                       updated = true;
                       storeCount++;
                   }
				}

               current.storeStartIndex ++;

               if (storeCount>=4)
                 return false;
			}

           
           /* Show pullup icon */
           if (!updated) {
               storeListView.listScroller.refresh();
              
           }

       }else {					/* Some error has occured */
			/* ** Need Exception Handler here */
			jAlert("We could not find the store you specified.", "Please Try Again");
		}
	},
	
	fetchDataCallBack: function (data, self) {
        
		/* reset the collection */
		self.reset();
      //  var storeCount = 0;
        this.storeStartIndex = 0;
        var current = this;
        console.log("fetchDataCallBack, data length:" + data.Store.length);
//		alert(JSON.stringify(data));
		/* populate data */
		if(data.Success == 1){	/* Success on StoreList webservice call */ 
			$.each(data.Store, function(index, value) {
				/* Filter the store and pharmacy hours */
				data.Store[index].StoreServices=data.Store[index].StoreServices.sort();
					
				/* Filter the store and pharmacy hours */
				if(storeFilterView.serviceList.indexOf("Stores Currently OPEN") >= 0
				&& storeFilterView.serviceList.indexOf("Pharmacies Currently OPEN") >= 0){
					/* Search for Both open */
					if(self.storeIsOpen(value) && self.pharmacyIsOpen(value)  && self.IsStoreHasFilterServices(data.Store[index].StoreServices)){
						/* Create model and set it's collection to self */
                       
						self.create(value).setCollection(self);
                    //    storeCount++;
                       
					}
				}else if(storeFilterView.serviceList.indexOf("Stores Currently OPEN") >= 0){
					/* Search for store open */
					if(self.storeIsOpen(value) && self.IsStoreHasFilterServices(data.Store[index].StoreServices)){
                        
						/* Create model and set it's collection to self */
						self.create(value).setCollection(self);
                    //    storeCount++;
                       
					}
				}else if(storeFilterView.serviceList.indexOf("Pharmacies Currently OPEN") >= 0){
					/* Search for store open */
					if(self.pharmacyIsOpen(value) && self.IsStoreHasFilterServices(data.Store[index].StoreServices)){
                        
						/* Create model and set it's collection to self */
						self.create(value).setCollection(self);
                     //   storeCount++;
					}
				}else{
					/* Create model and set it's collection to self */
                    if (self.IsStoreHasFilterServices(data.Store[index].StoreServices))
                    {
					    self.create(value).setCollection(self);
                      //  storeCount++;
                    }
				}

                current.storeStartIndex ++;
//                if (storeCount>=4)
//                    return false;
			});
			/* Check response length */
			if(self.length <= 0){
				if(location.hash=="#enrolment-request-page"){
		    		$("#enrollment-form-to-select").hide();
		    	}
				if(location.hash=="#transfer-page"){
		    		$("#transfer-form-to-seperator2").hide();
	                $("#transfer-form-to-select").hide();
		    	}
			     if(location.hash=="#refill-page"){
						$("#refill-page-seperator2").hide();
						$("#refill-form-location-result").hide();
			     }
				
				jAlert("Cannot find any store at this location.", "Please Try Again");
                if (storeLocatorView.fromPage == "filterSearch")
                    appView.searchFilter();
             }else{
				if(location.hash=="#enrolment-request-page"){
					$("#enrollment-form-to-select").show();
				}
				if(location.hash=="#transfer-page"){
				   $("#transfer-form-to-seperator2").show(200);
				   $("#transfer-form-to-select").show(200);
				}
				if(location.hash=="#refill-page"){
					$("#refill-page-seperator2").show();
					$("#refill-form-location-result").show();
				 }
				storeListMapView.showMap();
             }

		}else{					/* Some error has occured */
			if(location.hash=="#enrolment-request-page"){
	    		$("#enrollment-form-to-select").hide();
	    	}

			if(location.hash=="#transfer-page"){
	    		$("#transfer-form-to-seperator2").hide();
                $("#transfer-form-to-select").hide();
			 }
			if(location.hash=="#refill-page"){
				$("#refill-page-seperator2").hide();
				$("#refill-form-location-result").hide();
			}
			/* ** Need Exception Handler here */
			jAlert("No store found.", "Please Try Again");
		}
	
	},

	/*
	 * Success Call back function for phoneGap geolocation call
	 */
	phoneGapSuccess: function (position) {
		
		/* Remove spinner */
		$("#locator-page").find(".rexall-page-header").html("<p>STORE LOCATOR</p><div class='rexall-page-header-info'></div>");
		
		/* Remember the time of this successfully refresh */
		storeDetailList.lastRefresh = new Date();
		/* Hide spinner */
//		$("#locator-geo-search-load").css("visibility", "hidden");
//		$("#locator-search-input").attr("readOnly",false);
		//alert("Success! Populating stores...");
        console.log("Successfully get geolocation: " +JSON.stringify(position));
		storeDetailList.fetchDataWithLocation(position.coords.latitude, position.coords.longitude, 4);
	},
	
	/*
	 * Error Call back function for phoneGap geolocation call 
	 */
	phoneGapError: function (error) {
		
		/* Remove spinner */
		$("#locator-page").find(".rexall-page-header").html("<p>STORE LOCATOR</p><div class='rexall-page-header-info'></div>");
		
		var errorMsg;
		
		if(error.code == error.PERMISSION_DENIED){
			/* Permission Denied * Depending on platform */
			//errorMsg = "Turn on Location Services to allow Rexall to determine your Location";
			errorMsg = "Unable to determine your location. Please check your phone settings.";
		}else if(error.code == error.POSITION_UNAVAILABLE){
			/* Position not available */
			//errorMsg = "Turn on Location Services to allow Rexall to determine your Location";
			errorMsg = "Unable to determine your location. Please check your phone settings.";
		}else{
			/* Time out */
			errorMsg = "We are not able to determine your location, enter your postal code or try again later";
		}
		
		/* Show message */
		jAlert(errorMsg, "Location Service");
		
		/* Search with a default location */
		//storeDetailList.fetchData();
	},
	
	//url: "http://api.rexall.ca/v1/Rexall.svc/StoreList?APIKey=Ni878Yb4UjMef91G&latitude=43.6317&longitude=-79.6756&NumStores=5",
	url: getRexallWebserviceRoot(),
	
	/*
	 * Fetch data with a fix location, now set as Britannia's head office
	 */
	fetchData: function() {
		
		/* Set the search center of this collection */
		this.lat = 43.6317;
		this.lng = -79.6756;
		
		/* Construct web service options */
		var webServiceOption = {
			"webServiceURL"	: getRexallWebserviceRoot() + "/v3/standard/Rexall.svc/StoreList",
			"parameter"		: {
								"APIKey" 	: getWebserviceAPIKey(),	
								"latitude"	: 43.6317, 
								"longitude"	: -79.6756,
								//"NumStores"	: 5,
								//"NumStores"	: 4,
								"NumResults "	: 4,
							  },
			"callBack"		: this.fetchDataCallBack,		/* Call back function when webservice respond */
			"caller"		: this							/* refer back to the collection itself */
		};
		
		//this.fetch();
		
		/* Create new WebService instance */
		var webService = new WebService(webServiceOption);
		webService.callWebService();
	},

	/*
	 * Fetch data with a specific location
	 */
	fetchDataWithLocation: function(lat, lng, num) {
		console.log("Start fetching store data with location.... ("+lat+", "+lng+")");
		/* !! IMPORTANT
		 * Have to Remeber the storeFilterView.serviceList here for this search, 
		 * since in the callback for this search the serviceList will be reset and fetchMore will not behave correctly. */
		this.fetchMoreServiceList = storeFilterView.serviceList ? storeFilterView.serviceList : "";
		
		/* Set the search center of this collection */
		this.lat = lat;
		this.lng = lng;
		
		 var serviceFilter = this.GetFilterAttributeList();
       
       // console.log("fetchDataWithDefaultLocation, serviceFilter:" + JSON.stringify(serviceFilter) + " Para:" +JSON.stringify(para) );

        var self = this;
        var data =  {
			
						APIKey 	: getWebserviceAPIKey(),	
						latitude	: this.lat, 
						longitude	: this.lng,
						NumResults	: num ? num : 4,
						//"NumStores"	: num ? num : 4,
						FilterAttributeList : serviceFilter,
							 
		};

       /* Show spinner */
		$("#global-spinner").css("visibility", "visible");
		$("#locator-search-input").attr("readOnly",true);

        $.ajax({
                url: getRexallWebserviceRoot() + "v3/standard/Rexall.svc/StoreList",
                type: 'GET',
                data: data,
                crossDomain: true,
                dataType: 'json',
                success: function(res) {
                    console.log("Fetch data success, data: " + JSON.stringify(res));
                    /* Hide spinner */
		            $("#global-spinner").css("visibility", "hidden");
		            $("#locator-search-input").attr("readOnly",false);
                    self.fetchDataCallBack(res, self);
                    
                },
                error: function(XHR, textStatus, error) {
                    console.log("Fetch data error: " + JSON.stringify(XHR) + "\n" +textStatus +"\n" + error + "\n");
                     /* Hide spinner */
		            $("#global-spinner").css("visibility", "hidden");
		            $("#locator-search-input").attr("readOnly",false);
                    jAlert("An error occurs when trying to get the data, please try again.", "Rexall");
                },
                complete:function(XHR, textStatus) {
                    // self.afterFetch();

                }
            }); 
		
	},
	
	/*
	 * Fetch data with default location stored in this model
	 * For now I'm assuming this location is geolocation, but later on can be use with smarter method
	 */
	fetchDataWithDefaultLocation: function(num) {
		
		/* !! IMPORTANT
		 * Have to Remeber the storeFilterView.serviceList here for this search, 
		 * since in the callback for this search the serviceList will be reset and fetchMore will not behave correctly. */
		this.fetchMoreServiceList = storeFilterView.serviceList ? storeFilterView.serviceList : "";
		
		/* Check if default location exist */
		if(!this.lat || !this.lng || !this.hasUpdatedLocation()){
			this.fetchDataWithGeo();
			return;
		}
				
		
        var serviceFilter = this.GetFilterAttributeList();
       
       // console.log("fetchDataWithDefaultLocation, serviceFilter:" + JSON.stringify(serviceFilter) + " Para:" +JSON.stringify(para) );

        var self = this;
        var data =  {
			
						APIKey 	: getWebserviceAPIKey(),	
						latitude	: this.lat, 
						longitude	: this.lng,
						NumResults	: num ? num : 4,
						//"NumStores"	: num ? num : 4,
						FilterAttributeList : serviceFilter,
							 
		};

        console.log("fetchDataWithDefaultLocation data:" + JSON.stringify(data));

         /* Show spinner */
		$("#global-spinner").css("visibility", "visible");
		$("#locator-search-input").attr("readOnly",true);

        $.ajax({
                url: getRexallWebserviceRoot() + "v3/standard/Rexall.svc/StoreList",
                type: 'GET',
                data: data,
                crossDomain: true,
                dataType: 'json',
                success: function(res) {	
                     /* Hide spinner */
		            $("#global-spinner").css("visibility", "hidden");
		            $("#locator-search-input").attr("readOnly",false);
                    self.fetchDataCallBack(res, self);
                },
                error: function(XHR, textStatus, error) {
                     /* Hide spinner */
		            $("#global-spinner").css("visibility", "hidden");
		            $("#locator-search-input").attr("readOnly",false);
                    jAlert("An error occurs when trying to get the data, please try again.", "Rexall");
                },
                complete:function(XHR, textStatus) {
                    // self.afterFetch();
                }
            })

             

//		/* Construct web service options */
//		var webServiceOption = {
//			"webServiceURL"	: getRexallWebserviceRoot() + "/v2/standard/Rexall.svc/StoreList",
//			"parameter"		: {
//								"APIKey" 	: "Ni878Yb4UjMef91G",	
//								"latitude"	: this.lat, 
//								"longitude"	: this.lng,
//								"NumResults"	: num ? num : 12,
//								// "StoreServices.ServiceName" : para ? para : "",
//								"FilterAttributeList" : "[{ \"StoreServices.ServiceName\": para ? para : \"}]",
//							  },
//			"callBack"		: this.fetchDataCallBack,		/* Call back function when webservice respond */
//			"caller"		: this							/* refer back to the collection itself */
//		};
//		
//		/* Create new WebService instance */
//		var webService = new WebService(webServiceOption);
//		webService.callWebService();
		//storeFilterView.serviceList =[];
		
	},
	

	/*
	 * Fetch data with geolocation of the device
	 */
	fetchDataWithGeo:function () {
		
		/* Check wether the use has successfully located himself already */
		if(this.hasUpdatedLocation())
			return;
		
		
		var maximumAge = 5000;				/* Maxium age for cache of location, (PhoneGap function parameter) */
		var timeout = 15000;				/* Maxium time for sensing geolocation signal, (PhoneGap function parameter) */
		
		/* Add spinner */
		$("#locator-page").find(".rexall-page-header").html("Detecting your location &nbsp;&nbsp <div id='circleG'><div id='circleG_1' class='circleG'></div><div id='circleG_2' class='circleG'></div><div id='circleG_3' class='circleG'></div></div>");
		
		/* PhoneGap call */
		navigator.geolocation.getCurrentPosition(this.phoneGapSuccess, this.phoneGapError,  {maximumAge: maximumAge, timeout:timeout, enableHighAccuracy:true});
		
	},
	
	/*
	 * Fetch data with a input location
	 * Get lat and lng using google api first 
	 * localStorage.setItem("CurrentStoreListItemView", storeListItemView);
	 */
	fetchDataWithInput:function (q) {
		/* Reset geo search timer */
		this.lastRefresh = "";
		/* Search location */
		if(q){
			 var geocoder = new google.maps.Geocoder();
			 var self = this;
			 geocoder.geocode( { 'address': q}, function(results, status) {
				    if (status == google.maps.GeocoderStatus.OK) { 
				      var searchLat = results[0].geometry.location.lat();
				      var searchLng = results[0].geometry.location.lng();

				      /* Search this sepecific location */
				      //self.fetchDataWithLocation(searchLat, searchLng, 5);
				      self.fetchDataWithLocation(searchLat, searchLng, 30);
				      
				       /* reset my location */
				      storeListMapView.myLocation = new google.maps.LatLng(searchLat,searchLng);
				      storeListMapView.collection.lat=searchLat;
	    			  storeListMapView.collection.lng=searchLng;
	            	  storeListMapView.collection.reset();
				      
				    } else {
				      //jAlert('Geocode was not successful for the following reason: ' + status, "Error");
				    	jAlert('We were not able to find your location. Please try again.', "Something Went Wrong");
				    }
				  });
		}else{
			jAlert("Provide a valid address.", "Please Try Again");
			return true;
		}
	},
	
	fetchLocatorDataWithInput:function (q) {
		/* Reset geo search timer */
		this.lastRefresh = "";
		/* Search location */
		if(q){
			 var geocoder = new google.maps.Geocoder();
			 var self = this;
			 geocoder.geocode( { 'address': q}, function(results, status) {
				    if (status == google.maps.GeocoderStatus.OK) { 
				      var searchLat = results[0].geometry.location.lat();
				      var searchLng = results[0].geometry.location.lng();

				      /* Search this sepecific location */
				      //self.fetchDataWithLocation(searchLat, searchLng, 5);
				      self.fetchDataWithLocation(searchLat, searchLng, 4);
				      
				       /* reset my location */
				      storeListMapView.myLocation = new google.maps.LatLng(searchLat,searchLng);
				      storeListMapView.collection.lat=searchLat;
	    			  storeListMapView.collection.lng=searchLng;
	            	  storeListMapView.collection.reset();
				      
				    } else {
				      //jAlert('Geocode was not successful for the following reason: ' + status, "Error");
				    	jAlert('We were not able to find your location. Please try again.', "Something Went Wrong");
				    }
				  });
		}else{
			jAlert("Provide a valid address.", "Please Try Again");
			return true;
		}
	},

	/*
	 * Fetch data with input phone number 
	 * the response of this call should have only one store
	 */
	fetchDataWithPhone : function (phone) {

		/* Construct web service options */
		var webServiceOption = {
			"webServiceURL"	: getRexallWebserviceRoot() + "v3/standard/Rexall.svc/StoreGet/",
			"parameter"		: {
								"APIKey" 	: getWebserviceAPIKey(),
								"PhoneNumber": phone
							  },
			"callBack"		: this.fetchDataCallBack,		/* Call back function when webservice respond */
			"caller"		: this							/* refer back to the collection itself */
		};
		
		//this.fetch();
		
		/* Create new WebService instance */
		var webService = new WebService(webServiceOption);
		webService.callWebService();
	},
	
	
	/* 
	 * Fetch more stores in addition to the previous search 
	 * with the same search location
	 */
	fetchMore: function() {
//alert("fetch filter: " + this.fetchMoreServiceList.toString());
		
		var serviceFilter = this.GetFilterAttributeList();
        var self = this;
        var data =  {
			
						APIKey 	: getWebserviceAPIKey(),	
						latitude	: this.lat, 
						longitude	: this.lng,
						NumResults	: this.length + 4,
						//"NumStores"	: num ? num : 4,
						FilterAttributeList : serviceFilter,
							 
		};

        console.log("data:" + JSON.stringify(data));
         /* Show spinner */
		$("#global-spinner").css("visibility", "visible");
		$("#locator-search-input").attr("readOnly",true);

        $.ajax({
                url: getRexallWebserviceRoot() + "v3/standard/Rexall.svc/StoreList",
                type: 'GET',
                data: data,
                crossDomain: true,
                dataType: 'json',
                success: function(res) {	
                     /* Hide spinner */
		            $("#global-spinner").css("visibility", "hidden");
		            $("#locator-search-input").attr("readOnly",false);
                    self.fetchMoreCallBack(res, self);
                },
                error: function(XHR, textStatus, error) {
                    //alert(JSON.stringify(XHR) + "\n" +textStatus +"\n" + error + "\n");
                     /* Hide spinner */
		            $("#global-spinner").css("visibility", "hidden");
		            $("#locator-search-input").attr("readOnly",false);
                    jAlert("An error occurs when trying to get the data, please try again.", "Rexall");
                },
                complete:function(XHR, textStatus) {
                    // self.afterFetch();
                }
            }); 

//		/* Construct web service options */
//		var webServiceOption = {
//			"webServiceURL"	: getRexallWebserviceRoot() + "/v2/standard/Rexall.svc/StoreList",
//			"parameter"		: {
//								"APIKey" 	: "Ni878Yb4UjMef91G",	
//								"latitude"	: this.lat, 
//								"longitude"	: this.lng,
//								//"NumStores"	: this.length + 5,
//								"NumResults"	: this.length > 3 ? this.length + 12 : (this.length + 6) * 2,
//								/* IMPORTANT: fetchMore need to be consistant with original search
//								 * in terms of filter list.
//								 */
//								//"StoreServices.ServiceName" : para ? para : "",
//								//"FilterAttributeList" : "[{ \"StoreServices.ServiceName\": para ? para : \"}]",
//							  },
//			"callBack"		: this.fetchMoreCallBack,		/* Call back function when webservice respond */
//			"caller"		: this							/* refer back to the collection itself */
//		};
//		
//		/* Create new WebService instance */
//		var webService = new WebService(webServiceOption);
//		webService.callWebService();
	},

	
	/* Check wether the location is up to date, return true if the current location is up to date */
	hasUpdatedLocation: function() {
		var gap = 5,			/* 5 mins threshold */
			now = new Date();	/* Current search time */
		//alert("last : " + this.lastRefresh + "\nnow  :" + now + "\ngap  :" + (this.lastRefresh - now) / 60000);
		if(this.lastRefresh && ((now - this.lastRefresh) / 60000) < gap){
			//alert("is up to date");
			return true;
		}else{
			//alert("need update");
			return false;
		}
	},
	
	/* Check wether the input store is open or not */
	storeIsOpen:function(store){
		
		/* Get today */
		var today = new Date();
		var day = today.getDay();
		
		var storeHours = store.StoreHours;
		
		/* Start and End time of the day */
		var start, end;
		
		switch(day)
			{
			case 1:
				if(storeHours.MondayOpen){
					start = storeHours.MondayOpen;
					end = storeHours.MondayClose;
				}else{
					return false;
				}
				break;
			case 2:
				if(storeHours.TuesdayOpen){
					start = storeHours.TuesdayOpen;
					end = storeHours.TuesdayClose;
				}else{
					return false;
				}
				break;
			case 3:
				if(storeHours.WednesdayOpen){
					start = storeHours.WednesdayOpen;
					end = storeHours.WednesdayClose;
				}else{
					return false;
				}
				break;
			case 4:
				if(storeHours.ThursdayOpen){
					start = storeHours.ThursdayOpen;
					end = storeHours.ThursdayClose;
				}else{
					return false;
				}
				break;
			case 5:
				if(storeHours.FridayOpen){
					start = storeHours.FridayOpen;
					end = storeHours.FridayClose;
				}else{
					return false;
				}
				break;
			case 6:
				if(storeHours.SaturdayOpen){
					start = storeHours.SaturdayOpen;
					end = storeHours.SaturdayClose;
				}else{
					return false;
				}
				break;
			case 0:
				if(storeHours.SundayOpen){
					start = storeHours.SundayOpen;
					end = storeHours.SundayClose;
				}else{
					return false;
				}
				break;
			default:
				return false;
			}
		
		/* Determine wether it's open now */
		return this.getOpen(start, end);
	},
	
	/* Check wether the input pharmacy is open or not */
	pharmacyIsOpen:function(store){
		/* Get today */
		var today = new Date();
		var day = today.getDay();
		
		var storeHours = store.RxHours;
		
		/* Start and End time of the day */
		var start, end;
		
		switch(day)
			{
			case 1:
				if(storeHours.MondayOpen){
					start = storeHours.MondayOpen;
					end = storeHours.MondayClose;
				}else{
					return false;
				}
				break;
			case 2:
				if(storeHours.TuesdayOpen){
					start = storeHours.TuesdayOpen;
					end = storeHours.TuesdayClose;
				}else{
					return false;
				}
				break;
			case 3:
				if(storeHours.WednesdayOpen){
					start = storeHours.WednesdayOpen;
					end = storeHours.WednesdayClose;
				}else{
					return false;
				}
				break;
			case 4:
				if(storeHours.ThursdayOpen){
					start = storeHours.ThursdayOpen;
					end = storeHours.ThursdayClose;
				}else{
					return false;
				}
				break;
			case 5:
				if(storeHours.FridayOpen){
					start = storeHours.FridayOpen;
					end = storeHours.FridayClose;
				}else{
					return false;
				}
				break;
			case 6:
				if(storeHours.SaturdayOpen){
					start = storeHours.SaturdayOpen;
					end = storeHours.SaturdayClose;
				}else{
					return false;
				}
				break;
			case 0:
				if(storeHours.SundayOpen){
					start = storeHours.SundayOpen;
					end = storeHours.SundayClose;
				}else{
					return false;
				}
				break;
			default:
				return false;
			}
		
		/* Determine wether it's open now */
		return this.getOpen(start, end);
	},
	
	/* Determine wether a store is currently open during the input period */
	getOpen: function(start, end){
		if(start < 0){
			/* if not open at all */
			return false;
		}
		
		/* Store end time */
		var finalEnd = end;
		
		start = start.substr(0, start.length-2);
		end = end.substr(0, end.length-2);
		
		var s = start.split(":");
		var e = end.split(":");
	
		/* Create date with arbitrary year and month to compare time */
		var ss=new Date(parseInt("2001"),(parseInt("01"))-1,parseInt("01"),parseInt(s[0]),parseInt(s[1]),parseInt("0"));
		/* Afternoon time, need to revise parse ! */
		var ee=new Date(parseInt("2001"),(parseInt("01"))-1,parseInt("01"),parseInt(e[0]) + 12,parseInt(e[1]),parseInt("0"));
		var current = new Date();
		var cc = new Date(parseInt("2001"),(parseInt("01"))-1,parseInt("01"),current.getHours(), current.getMinutes(),parseInt("0"));
		
		/* Output result */
		if(ss.valueOf() < cc.valueOf() && cc.valueOf() < ee.valueOf()){
			return true;	/* open now */
		}else{
			return false; 	/* close now */
		}
	},
	
    IsStoreHasFilterServices : function(storeServices) {
        //As web service will do the filtering, always return true here
        return true;

        var i = 0;
		for(i = 0; i < storeFilterView.serviceList.length; i++){

            var filterService = storeFilterView.serviceList[i].toString();

            if( filterService.indexOf("Stores Currently OPEN") >= 0
				|| filterService.indexOf("Pharmacies Currently OPEN") >= 0 )
                {
                 //not a real service, ignored
                }
            else
            {
			    if(storeServices.indexOf(filterService)<=0)
				 return false;
            }
		}

        return true;
    },

    GetFilterAttributeList : function() {
        /* construct parameters, need to remove store hours & pharmacy hours */
		if(storeFilterView.serviceList){
			var startIndex = Math.max(storeFilterView.serviceList.indexOf("Pharmacies Currently OPEN"), storeFilterView.serviceList.indexOf("Stores Currently OPEN"));
			var para = storeFilterView.serviceList.slice(startIndex + 1, storeFilterView.serviceList.length);
		}

        var serviceFilter = [];
        for (var i=0; i<para.length; i++)
        {
            var service = {"StoreServices.ServiceName" : para[i].toString()};
           // console.log(JSON.stringify(service));
            serviceFilter.push(service);
        }

        return serviceFilter;
    }

});