var ServiceView = Backbone.View.extend({
		
	initialize: function (options) {
		
		/* Add self to service view */
		options.serviceView.services.push(this);
		/* Set search filter */
		this.filter = options.filter;	
		
		/* Create new collection for this view */
	    this.collection = new StoreDetailList();
		this.listenTo(this.collection, "add", this.addOne);	/* Listen to collection of this view */
		this.listenTo(this.collection, "reset", this.cleanup);	/* Listen to collection of this view */
	},
	
	events: {
		"tap .service-list-header" 			: "openOrClose",
		"tap .service-list-close-button"	: "openOrClose",
		"tap .service-list-submit"			: "submitQuery",
		"keypress .service-list-input"		: "submit",
	},
	
	open: function () {
		this.$el.removeClass("service-list-close").addClass("service-list-open");
	},
	
	close: function () {
		this.$el.removeClass("service-list-open").addClass("service-list-close");
	},
	
	openOrClose: function() {
		if(this.$el.attr("class") == "service-list-close"){
			this.open();
		}else{
			this.close();
		}		
	},
	
	submit: function (e) {
		if (e.keyCode != 13) return;
		
		this.submitQuery();
	},
	
	submitQuery: function() {
		var q = this.$el.find(".service-list-input").val();
		var self = this;
		if(q){
			 geocoder = new google.maps.Geocoder();
			 geocoder.geocode( { 'address': q}, function(results, status) {
				    if (status == google.maps.GeocoderStatus.OK) { 
				      /* Search this sepecific location */
				    	self.populateStores(results);			      				      
				    } else {
				      alert('Geocode was not successful for the following reason: ' + status);
				      return false;
				    }
				  });
		}else{
			jAlert("Provide a valid address.","Please Try Again");
			return false;
		}
		
				
	},
	
	populateStores: function(results) {
		var lat = results[0].geometry.location.lat();
	    var lng = results[0].geometry.location.lng();
	    
	    /* remove original stores */
	    this.collection.reset();
	    
	    /* fetch data */
	    if(this.filter === "Medical Clinic")
	    {
	    	this.collection.fetchDataWithLocation(lat, lng, 400);
	    }else{
	    	this.collection.fetchDataWithLocation(lat, lng);
	    }
	},
	
	addOne: function (item) {
		/* First need to filter out the stores without the services */
		if(item.toJSON().StoreServices.indexOf(this.filter) >= 0){
			var serviceStoreView = new ServiceStoreView({model : item});
			this.$el.find(".service-list-content").children("ul").append(serviceStoreView.render().el);
		}
	},
	
	cleanup: function () {
		/* Remove the history of previews search */
		this.$el.find(".service-list-content").children("ul").html("");
	}
	
		
});