var ShoppingListView = Backbone.View.extend({
    el: $("#shopping-page"),
    MAX_INPUT_LENGTH: 27,   // maximum number of characters a use could input
    //statsTemplate: _.template($('#stats-template').html()),
    events: {
        "keypress #shopping-input"				:  	"createOnEnter",
        "tap #shopping-input-add"					: 	"createOnTap",
        "tap #shopping-manage-list-switch"		:	"toggleManageList",	
        "tap #shopping-input"		:	"toggleManageListHide",	

        //"click #clear-completed": "clearCompleted",
        //"click #toggle-all": "toggleAllComplete"
    },
    initialize: function() {
        this.input = this.$("#shopping-input");
        //this.allCheckbox = this.$("#toggle-all")[0];
      
        /* Initialize shopping scroller */
        this.shoppingListScroller = new iScroll("shopping-list-container", {
            hScroll:false, 
            vScroll:true, 
            vScrollbar: true, 
            hScrollbar: false, 
            fadeScrollbar: true, 
            hideScrollbar:true
        });
      
        /* Listen to collection event */
        this.listenTo(this.collection, 'add', this.addOne);
        //this.listenTo(this.collection, 'reset', this.addAll);
        this.listenTo(this.collection, 'all', this.render);
      
        /* Ratio button change */
        var self = this;
        $("input[name='shopping-list-menu']").change(function() {
            self.menuOperation();
        });
      
        /* Fetch existing shopping list */
        this.collection.fetch();
    },
    
    menuOperation: function() {
        var op = $("input[name='shopping-list-menu']:checked").val();
        var self = this;
        if (op === "showIncomplete") {
            this.showIncomplete();
            this.toggleManageList2();
        }
        else if (op === "showComplete") {
            this.showComplete();
            this.toggleManageList2();
        }
        else if (op === "deleteIncomplete") {
            /* Check length of target list */
            if (this.collection.notDone().length <= 0) {
                jAlert("There is no item to delete", "Please Try Again");
                self.toggleManageList2();
                return false;
            }
            jConfirm('Delete all incomplete items?', 'Please Confirm', function(r) {
                if (r) {
                    shoppingListView.deleteIncomplete();
                    self.toggleManageList2();
                }
                else {
                    self.toggleManageList2();
                    return false;
                }
            })           		
        }
        else if (op === "deleteComplete") {
            /* Check length of target list */
            if (this.collection.done().length <= 0) {
                jAlert("There is no item to delete", "Please Try Again");
                self.toggleManageList2();
                return false;
            }
            jConfirm('Delete all completed items?', 'Please Confirm', function(r) {
                if (r) {
                    shoppingListView.deleteComplete();
                    self.toggleManageList2();
                }
                else {
                    self.toggleManageList2();
                    return false;
                }
            }) 
        }
        else if (op === "deleteAll") {
            /* Check length of target list */
            if (this.collection.length <= 0) {
                jAlert("There is no item to delete", "Please Try Again");
                self.toggleManageList2();
                return false;
            }
            jConfirm('Delete all items?', 'Please Confirm', function(r) {
                if (r) {
                    shoppingListView.deleteAll();
                    self.toggleManageList2();
                }
                else {
                    self.toggleManageList2();
                    return false;
                }
            })  
        }
        else if (op === "showAll") {
            this.showAll();
            this.toggleManageList2();
        }
        else {
            // Error operation, do nothing
        }
    },
    
    toggleManageList: function(e) {
        e.preventDefault();
        $("#shopping-manage-list-menu").toggle(200);
        var menu = $("input[name='shopping-list-menu']:checked");
        $(menu).prop('checked', false);
    },
    
    toggleManageList2: function() {
        $("#shopping-manage-list-menu").toggle(400);
    },

     toggleManageListHide: function() {
        if ($("#shopping-manage-list-menu").is(":visible"))
            $("#shopping-manage-list-menu").toggle(200);
    },

    reRenderListItem: function() {
        var model;
        for (var i = 0; i < this.collection.length; i++) {
            model = this.collection.models[i];
            model.toggle();
            model.toggle();
        }
    },
    
    addOne: function(item) {
        var view = new ShoppingItemView({model: item});
        this.$("#shopping-list").append(view.render().el);
        /* refresh scroller */
        this.shoppingListScroller.refresh();
    },
    //    addAll: function() {
    //      Todos.each(this.addOne, this);
    //    },
    render : function(){
    	if($("#shopping-list li").length>0){
    		$("#shopping-list-divider").show();
    	}else{
    		$("#shopping-list-divider").hide();
    	}
    },
    createOnTap: function() {
        if (!this.input.val())
            return;
        this.collection.create({title: this.input.val().substring(0, this.MAX_INPUT_LENGTH)});
        this.input.val('');
    },
    createOnEnter: function(e) {
        if (e.keyCode != 13)
        {   var inputval = this.input.val();
            if (inputval.length >= this.MAX_INPUT_LENGTH)
            {   inputval = inputval.substring(0, this.MAX_INPUT_LENGTH);
                var self = this;
                setTimeout(function()
                {self.input.val(inputval);},300);
            }
            return;
         }
        this.createOnTap();
    },
    
    showIncomplete:function() {
    	var self = this;
        this.$el.find("#shopping-list").find("li").each(function(index, value) {
            //alert($(value).html());
            if ($(value).find(".shopping-toggle").attr("checked")) {
                /* Completed */
                $(value).hide(200, function() {
                    /* refresh scroller */
                    self.shoppingListScroller.refresh();
                });
            }
            else {
                /* inCompleted */
                $(value).show(200, function() {
                    /* refresh scroller */
                    self.shoppingListScroller.refresh();
                });
            }
        });
        /* refresh scroller */
        //        this.shoppingListScroller.refresh();
    },
    
    showComplete: function() {
    	var self = this;
        this.$el.find("#shopping-list").find("li").each(function(index, value) {
            //alert($(value).html());
            if ($(value).find(".shopping-toggle").attr("checked")) {
                /* Completed */
                $(value).show(200, function() {
                    /* refresh scroller */
                    self.shoppingListScroller.refresh();
                });
            }
            else {
                /* inCompleted */
                $(value).hide(200, function() {
                    /* refresh scroller */
                    self.shoppingListScroller.refresh();
                });
            }
        });
        /* refresh scroller */
        //    	var self = this;
        //    	setTimeout(function(){
        //    		self.shoppingListScroller.refresh();
        //    	},1000);
    },
    
    showAll: function() {
        var self = this;
    	
        this.$el.find("#shopping-list").find("li").each(function(index, value) {
            $(value).show(200, function() {
                /* refresh scroller */
                self.shoppingListScroller.refresh();
            });
        });
    },
    
    deleteAll: function() {
        var model;
        while (model = this.collection.first()) {
            model.destroy();
        }
    },
    
    deleteComplete: function() {
        _.invoke(this.collection.done(), 'destroy');
        this.shoppingListScroller.refresh();
        return false;
    },
    
    deleteIncomplete: function() {
        console.log("deleteIncomplete");
        _.invoke(this.collection.notDone(), 'destroy');
        this.shoppingListScroller.refresh();
        return false;
    },
    //    toggleAllComplete: function () {
    //      var done = this.allCheckbox.checked;
    //      Todos.each(function (todo) { todo.save({'done': done}); });
    //    }
});